#!/usr/bin/env python

import unittest

import base64
import gflags
import os
import sys

from ct.crypto import cert
from ct.crypto import error
from ct.crypto import pem
from ct.crypto import verify
from ct.proto import client_pb2
from ct.serialization import tls_message
import mock

FLAGS = gflags.FLAGS
gflags.DEFINE_string("testdata_dir", "../test/testdata",
                     "Location of test certs")

SYMANTEC_B64_KEY = (
    'MFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAEluqsHEYMG1XcDfy1lCdGV0JwOmkY4r'
    '87xNuroPS2bMBTP01CEDPwWJePa75y9CrsHEKqAy8afig1dpkIPSEUhg=='
)

VENAFI_B64_KEY = (
    'MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAolpIHxdSlTXLo1s6H1OCdp'
    'Sj/4DyHDc8wLG9wVmLqy1lk9fz4ATVmm+/1iN2Nk8jmctUKK2MFUtlWXZBSpym97M7'
    'frGlSaQXUWyA3CqQUEuIJOmlEjKTBEiQAvpfDjCHjlV2Be4qTM6jamkJbiWtgnYPhJ'
    'L6ONaGTiSPm7Byy57iaz/hbckldSOIoRhYBiMzeNoA0DiRZ9KmfSeXZ1rB8y8X5urS'
    'W+iBzf2SaOfzBvDpcoTuAaWx2DPazoOl28fP1hZ+kHUYvxbcMjttjauCFx+JII0dmu'
    'ZNIwjfeG/GBb9frpSX219k1O4Wi6OEbHEr8at/XQ0y7gTikOxBn/s5wQIDAQAB'
)

def read_testdata_file(test_file):
    with open(os.path.join(FLAGS.testdata_dir, test_file), 'rb') as f:
        return f.read()


class LogVerifierTest(object):
    """Defines tests for STH and SCT verification logic.

    In order to run these tests, one or more derived classes must be created.
    These classes should also inherit from unittest.TestCase. The derived
    classes must define the following members for use by the tests:
    - self.key_info_fixture: A client_pb2.KeyInfo object
    - self.sth_fixture: A client_pb2.SthResponse object

    This is so that the tests can be run repeatedly with a variety of public
    keys and STHs (e.g. RSA, ECDSA).
    """

    def test_verify_sth(self):
        verifier = verify.LogVerifier(self.key_info_fixture)
        self.assertTrue(verifier.verify_sth(self.sth_fixture))

    def test_verify_sth_fails_for_bad_signature(self):
        verifier = verify.LogVerifier(self.key_info_fixture)
        sth_fixture = self.sth_fixture

        for i in range(len(sth_fixture.tree_head_signature)):
            # Skip the bytes that encode ASN.1 lengths: this is covered in a
            # separate test
            if i == 5 or i == 7 or i == 42:
                continue
            sth = client_pb2.SthResponse()
            sth.CopyFrom(sth_fixture)
            sth.tree_head_signature = (
                sth_fixture.tree_head_signature[:i] +
                chr(ord(sth_fixture.tree_head_signature[i]) ^ 1) +
                sth_fixture.tree_head_signature[i+1:])
            # Encoding- or SignatureError, depending on whether the modified
            # byte is a content byte or not.
            self.assertRaises((error.EncodingError, error.SignatureError),
                              verifier.verify_sth, sth)

    def test_verify_sth_consistency(self):
        old_sth = self.sth_fixture
        new_sth = client_pb2.SthResponse()
        new_sth.CopyFrom(old_sth)
        new_sth.tree_size = old_sth.tree_size + 1
        new_sth.timestamp = old_sth.timestamp + 1
        new_sth.sha256_root_hash = "a new hash"
        proof = ["some proof the mock does not care about"]

        mock_merkle_verifier = mock.Mock()
        mock_merkle_verifier.verify_tree_consistency.return_value = True

        verifier = verify.LogVerifier(self.key_info_fixture,
                                      mock_merkle_verifier)
        self.assertTrue(verifier.verify_sth_consistency(old_sth, new_sth,
                                                        proof))
        mock_merkle_verifier.verify_tree_consistency.assert_called_once_with(
            old_sth.tree_size, new_sth.tree_size, old_sth.sha256_root_hash,
            new_sth.sha256_root_hash, proof)

    def test_verify_sth_temporal_consistency(self):
        old_sth = self.sth_fixture
        new_sth = client_pb2.SthResponse()
        new_sth.CopyFrom(old_sth)
        new_sth.tree_size = old_sth.tree_size + 1
        new_sth.timestamp = old_sth.timestamp + 1

        # Merkle verifier is never used so simply set to None
        verifier = verify.LogVerifier(self.key_info_fixture,
                                      None)

        # Note we do not care about root hash inconsistency here.
        self.assertTrue(verifier.verify_sth_temporal_consistency(
            old_sth, new_sth))

    def test_verify_sth_temporal_consistency_equal_timestamps(self):
        old_sth = self.sth_fixture
        new_sth = client_pb2.SthResponse()
        new_sth.CopyFrom(old_sth)
        new_sth.tree_size = old_sth.tree_size + 1

        # Merkle verifier is never used so simply set to None
        verifier = verify.LogVerifier(self.key_info_fixture,
                                      None)

        self.assertRaises(error.ConsistencyError,
                          verifier.verify_sth_temporal_consistency,
                          old_sth, new_sth)

        new_sth.tree_size = old_sth.tree_size - 1
        self.assertRaises(error.ConsistencyError,
                          verifier.verify_sth_temporal_consistency,
                          old_sth, new_sth)

        # But identical STHs are OK
        self.assertTrue(verifier.verify_sth_temporal_consistency(
            old_sth, old_sth))

    def test_verify_sth_temporal_consistency_reversed_timestamps(self):
        old_sth = self.sth_fixture
        new_sth = client_pb2.SthResponse()
        new_sth.CopyFrom(old_sth)
        new_sth.timestamp = old_sth.timestamp + 1
        new_sth.tree_size = old_sth.tree_size + 1

        # Merkle verifier is never used so simply set to None
        verifier = verify.LogVerifier(self.key_info_fixture,
                                      None)

        self.assertRaises(ValueError,
                          verifier.verify_sth_temporal_consistency,
                          new_sth, old_sth)

    def test_verify_sth_temporal_consistency_newer_tree_is_smaller(self):
        old_sth = self.sth_fixture
        new_sth = client_pb2.SthResponse()
        new_sth.CopyFrom(old_sth)
        new_sth.timestamp = old_sth.timestamp + 1
        new_sth.tree_size = old_sth.tree_size - 1

        # Merkle verifier is never used so simply set to None
        verifier = verify.LogVerifier(self.key_info_fixture,
                                      None)

        self.assertRaises(error.ConsistencyError,
                          verifier.verify_sth_temporal_consistency,
                          old_sth, new_sth)

    def test_verify_sth_consistency_invalid_proof(self):
        old_sth = self.sth_fixture
        new_sth = client_pb2.SthResponse()
        new_sth.CopyFrom(old_sth)
        new_sth.tree_size = old_sth.tree_size + 1
        new_sth.timestamp = old_sth.timestamp + 1
        new_sth.sha256_root_hash = "a new hash"
        proof = ["some proof the mock does not care about"]

        mock_merkle_verifier = mock.Mock()
        mock_merkle_verifier.verify_tree_consistency.side_effect = (
            error.ConsistencyError("Evil"))

        verifier = verify.LogVerifier(self.key_info_fixture,
                                      mock_merkle_verifier)
        self.assertRaises(error.ConsistencyError,
                          verifier.verify_sth_consistency,
                          old_sth, new_sth, proof)

    def _test_verify_sct(self, proof, chain, fake_timestamp = None):
        sct = client_pb2.SignedCertificateTimestamp()
        tls_message.decode(read_testdata_file(proof), sct)
        if fake_timestamp is not None:
            sct.timestamp = fake_timestamp

        chain = map(lambda name: cert.Certificate.from_pem_file(
                        os.path.join(FLAGS.testdata_dir, name)), chain)

        key_info = client_pb2.KeyInfo()
        key_info.type = client_pb2.KeyInfo.ECDSA
        key_info.pem_key = read_testdata_file('ct-server-key-public.pem')

        verifier = verify.LogVerifier(key_info)
        return verifier.verify_sct(sct, chain)

    def _test_verify_embedded_scts(self, chain):
        chain = map(lambda name: cert.Certificate.from_pem_file(
                        os.path.join(FLAGS.testdata_dir, name)), chain)

        key_info = client_pb2.KeyInfo()
        key_info.type = client_pb2.KeyInfo.ECDSA
        key_info.pem_key = read_testdata_file('ct-server-key-public.pem')

        verifier = verify.LogVerifier(key_info)
        return verifier.verify_embedded_scts(chain)

    def test_verify_sct_valid_signature(self):
        self.assertTrue(self._test_verify_sct(
                                'test-cert.proof',
                                ['test-cert.pem', 'ca-cert.pem']))

    def test_verify_sct_invalid_signature(self):
        self.assertRaises(error.SignatureError,
                          self._test_verify_sct,
                          'test-cert.proof',
                          ['test-cert.pem', 'ca-cert.pem'],
                          fake_timestamp = 1234567)

    def test_verify_sct_precertificate_valid_signature(self):
        self.assertTrue(self._test_verify_sct(
                                'test-embedded-pre-cert.proof',
                                ['test-embedded-pre-cert.pem', 'ca-cert.pem']))

    def test_verify_sct_precertificate_invalid_signature(self):
        self.assertRaises(error.SignatureError,
                          self._test_verify_sct,
                          'test-embedded-pre-cert.proof',
                          ['test-embedded-pre-cert.pem', 'ca-cert.pem'],
                          fake_timestamp = 1234567)

    def test_verify_sct_precertificate_with_preca_valid_signature(self):
        self.assertTrue(self._test_verify_sct(
                                'test-embedded-with-preca-pre-cert.proof',
                                ['test-embedded-with-preca-pre-cert.pem',
                                 'ca-pre-cert.pem', 'ca-cert.pem']))

    def test_verify_sct_missing_leaf_cert(self):
        self.assertRaises(error.IncompleteChainError,
                          self._test_verify_sct,
                          'test-cert.proof',
                          [])

    def test_verify_sct_missing_issuer_cert(self):
        self.assertRaises(error.IncompleteChainError,
                          self._test_verify_sct,
                          'test-embedded-pre-cert.proof',
                          ['test-embedded-pre-cert.pem'])

    def test_verify_sct_with_preca_missing_issuer_cert(self):
        self.assertRaises(error.IncompleteChainError,
                          self._test_verify_sct,
                          'test-embedded-with-preca-pre-cert.proof',
                          ['test-embedded-with-preca-pre-cert.pem',
                           'ca-pre-cert.pem'])

    def test_verify_embedded_scts_valid_signature(self):
        sct = client_pb2.SignedCertificateTimestamp()
        tls_message.decode(read_testdata_file('test-embedded-pre-cert.proof'),
                           sct)

        result = self._test_verify_embedded_scts(
                    ['test-embedded-cert.pem', 'ca-cert.pem'])
        self.assertEqual(result, [(sct, True)])

    def test_verify_embedded_scts_invalid_signature(self):
        result = self._test_verify_embedded_scts(
                    ['test-invalid-embedded-cert.pem', 'ca-cert.pem'])
        self.assertFalse(result[0][1])

    def test_verify_embedded_scts_with_preca_valid_signature(self):
        sct = client_pb2.SignedCertificateTimestamp()
        tls_message.decode(
                read_testdata_file('test-embedded-with-preca-pre-cert.proof'),
                sct)

        result = self._test_verify_embedded_scts(
                    ['test-embedded-with-preca-cert.pem', 'ca-cert.pem'])
        self.assertEqual(result, [(sct, True)])


class LogVerifierRsaTest(LogVerifierTest, unittest.TestCase):
    sth_fixture = client_pb2.SthResponse()
    sth_fixture.tree_size = 1130
    sth_fixture.timestamp = 1442500998291
    sth_fixture.sha256_root_hash = (
        "58f4e84d26f179829da3359a23f2ec519f83e99d9230aad6bfb37e2faa82c663"
        ).decode("hex")

    sth_fixture.tree_head_signature = (
        "040101002595c278829d558feb560c5024048ce1ca9e5329cc79b074307f0b6168dda1"
        "5b27f84c94cce39f8371aa8205d73a7101b434b6aeaf3c852b8471daa05d654463b334"
        "5103c7406dbd4642c8cc89eababa84e9ad663ffb3cc87940c3689d0c2ac6246915f221"
        "5da254981206fed8505eed268bcc94e05cd83c8e8e5a14407a6d15c8071fabaed9728a"
        "02830c6aef95969b0576c7ae09d50bdfc8b0b58fa759458c6d62383d6fe1072c0da103"
        "1baddfa363b58ca78f93f329b1f1a15b9575988974dcba2421b9a1bb2a617d8b3f4046"
        "ead6095f8496075edc686ae4fa672d4974de0fb9326dc3c628f7e44c7675d2c56d1c66"
        "32bbb9e4a69e0a7e34bd1d6dc7b4b2").decode("hex")

    key_info_fixture = client_pb2.KeyInfo()
    key_info_fixture.type = client_pb2.KeyInfo.RSA
    key_info_fixture.pem_key = (
        "-----BEGIN PUBLIC KEY-----\n"
        "MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAolpIHxdSlTXLo1s6H1OC\n"
        "dpSj/4DyHDc8wLG9wVmLqy1lk9fz4ATVmm+/1iN2Nk8jmctUKK2MFUtlWXZBSpym\n"
        "97M7frGlSaQXUWyA3CqQUEuIJOmlEjKTBEiQAvpfDjCHjlV2Be4qTM6jamkJbiWt\n"
        "gnYPhJL6ONaGTiSPm7Byy57iaz/hbckldSOIoRhYBiMzeNoA0DiRZ9KmfSeXZ1rB\n"
        "8y8X5urSW+iBzf2SaOfzBvDpcoTuAaWx2DPazoOl28fP1hZ+kHUYvxbcMjttjauC\n"
        "Fx+JII0dmuZNIwjfeG/GBb9frpSX219k1O4Wi6OEbHEr8at/XQ0y7gTikOxBn/s5\n"
        "wQIDAQAB\n"
        "-----END PUBLIC KEY-----\n")


class LogVerifierEcdsaTest(LogVerifierTest, unittest.TestCase):
    sth_fixture = client_pb2.SthResponse()
    sth_fixture.tree_size = 42
    sth_fixture.timestamp = 1348589667204
    sth_fixture.sha256_root_hash = (
        "18041bd4665083001fba8c5411d2d748e8abbfdcdfd9218cb02b68a78e7d4c23"
        ).decode("hex")

    sth_fixture.tree_head_signature = (
        "040300483046022100befd8060563763a5e49ba53e6443c13f7624fd6403178113736e"
        "16012aca983e022100f572568dbfe9a86490eb915c4ee16ad5ecd708fed35ed4e5cd1b"
        "2c3f087b4130").decode("hex")

    key_info_fixture = client_pb2.KeyInfo()
    key_info_fixture.type = client_pb2.KeyInfo.ECDSA
    key_info_fixture.pem_key = (
        "-----BEGIN PUBLIC KEY-----\nMFkwEwYHKoZIzj0CAQYIKoZIzj0DAQcDQgAES0AfBk"
        "jr7b8b19p5Gk8plSAN16wW\nXZyhYsH6FMCEUK60t7pem/ckoPX8hupuaiJzJS0ZQ0SEoJ"
        "GlFxkUFwft5g==\n-----END PUBLIC KEY-----\n")

    def test_verify_sth_for_bad_asn1_length(self):
        verifier = verify.LogVerifier(self.key_info_fixture)
        sth_fixture = self.sth_fixture

        # The byte that encodes the length of the ASN.1 signature sequence
        i = 5

        # Decreasing the length truncates the sequence and causes a decoding
        # error.
        sth = client_pb2.SthResponse()
        sth.CopyFrom(sth_fixture)
        sth.tree_head_signature = (
            sth_fixture.tree_head_signature[:i] +
            chr(ord(sth_fixture.tree_head_signature[i]) - 1) +
            sth_fixture.tree_head_signature[i+1:])
        self.assertRaises(error.EncodingError, verifier.verify_sth, sth)

        # Increasing the length means there are not enough ASN.1 bytes left to
        # decode the sequence, however the ecdsa module silently slices it.
        # Our ECDSA verifier checks for it and will fail.
        sth = client_pb2.SthResponse()
        sth.CopyFrom(sth_fixture)
        sth.tree_head_signature = (
            sth_fixture.tree_head_signature[:i] +
            chr(ord(sth_fixture.tree_head_signature[i]) + 1) +
            sth_fixture.tree_head_signature[i+1:])
        self.assertRaises(
            error.EncodingError,
            verifier.verify_sth,
            sth)

        # The byte that encodes the length of the first integer r in the
        # sequence (r, s). Modifying the length corrupts the second integer
        # offset and causes a decoding error.
        i = 7
        sth = client_pb2.SthResponse()
        sth.CopyFrom(sth_fixture)
        sth.tree_head_signature = (
            sth_fixture.tree_head_signature[:i] +
            chr(ord(sth_fixture.tree_head_signature[i]) - 1) +
            sth_fixture.tree_head_signature[i+1:])
        self.assertRaises(error.EncodingError, verifier.verify_sth, sth)

        sth = client_pb2.SthResponse()
        sth.CopyFrom(sth_fixture)
        sth.tree_head_signature = (
            sth_fixture.tree_head_signature[:i] +
            chr(ord(sth_fixture.tree_head_signature[i]) + 1) +
            sth_fixture.tree_head_signature[i+1:])
        self.assertRaises(error.EncodingError, verifier.verify_sth, sth)

        # The byte that encodes the length of the second integer s in the
        # sequence (r, s). Increasing this length leaves bytes unread which
        # is now also detected in the verify_ecdsa module.
        i = 42
        sth = client_pb2.SthResponse()
        sth.CopyFrom(sth_fixture)
        sth.tree_head_signature = (
            sth_fixture.tree_head_signature[:i] +
            chr(ord(sth_fixture.tree_head_signature[i]) - 1) +
            sth_fixture.tree_head_signature[i+1:])
        self.assertRaises(error.EncodingError, verifier.verify_sth, sth)

        sth = client_pb2.SthResponse()
        sth.CopyFrom(sth_fixture)
        sth.tree_head_signature = (
            sth_fixture.tree_head_signature[:i] +
            chr(ord(sth_fixture.tree_head_signature[i]) + 1) +
            sth_fixture.tree_head_signature[i+1:])
        self.assertRaises(error.EncodingError, verifier.verify_sth, sth)

        # Trailing garbage is correctly detected.
        sth = client_pb2.SthResponse()
        sth.CopyFrom(sth_fixture)
        sth.tree_head_signature = (
            sth_fixture.tree_head_signature[:3] +
            # Correct outer length to include trailing garbage.
            chr(ord(sth_fixture.tree_head_signature[3]) + 1) +
            sth_fixture.tree_head_signature[4:]) + "\x01"
        self.assertRaises(error.EncodingError, verifier.verify_sth, sth)

    def test_verify_sth_for_bad_asn1_signature(self):
        # www.google.com certificate for which a bad SCT was issued.
        google_cert = (
            '-----BEGIN CERTIFICATE-----',
            'MIIEgDCCA2igAwIBAgIIdJ7+eILLLSgwDQYJKoZIhvcNAQELBQAwSTELMAkGA1UE',
            'BhMCVVMxEzARBgNVBAoTCkdvb2dsZSBJbmMxJTAjBgNVBAMTHEdvb2dsZSBJbnRl',
            'cm5ldCBBdXRob3JpdHkgRzIwHhcNMTUxMDA3MTExMDM4WhcNMTYwMTA1MDAwMDAw',
            'WjBoMQswCQYDVQQGEwJVUzETMBEGA1UECAwKQ2FsaWZvcm5pYTEWMBQGA1UEBwwN',
            'TW91bnRhaW4gVmlldzETMBEGA1UECgwKR29vZ2xlIEluYzEXMBUGA1UEAwwOd3d3',
            'Lmdvb2dsZS5jb20wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCR6Knj',
            'TG6eyvY6C1VO7daC0AbWe3cenr9y9lVFQH2ej5r87znUvep4pC/bmG71aTd25wds',
            'ScpclWNR4lkR9Ph45j8K+SjMXU7syiqFiWPWgVzyi4N3bXZw4w83RoTzfyUTn4Kx',
            '9nsQLmjVS4wUMSEpWBmYfORwUwMF8BYp5qSkIUogZTADPY7Qr8tmwEq8jLHv9z62',
            'SiYd9JEcGdhnajgXg/+/f+iIb1jhkbjsTjFJBHClgrtRqLZHSU1THZCK6iULTd1B',
            '4yBNvXcHDaSBTPUSvZvZXo/msKfOqd0fHtny1icgl5CSU0tZrZPteomMnLMGdLlN',
            'KHyqIX7XsAd3pNoXAgMBAAGjggFLMIIBRzAdBgNVHSUEFjAUBggrBgEFBQcDAQYI',
            'KwYBBQUHAwIwGQYDVR0RBBIwEIIOd3d3Lmdvb2dsZS5jb20waAYIKwYBBQUHAQEE',
            'XDBaMCsGCCsGAQUFBzAChh9odHRwOi8vcGtpLmdvb2dsZS5jb20vR0lBRzIuY3J0',
            'MCsGCCsGAQUFBzABhh9odHRwOi8vY2xpZW50czEuZ29vZ2xlLmNvbS9vY3NwMB0G',
            'A1UdDgQWBBSUPOkxr+tGC3JYs2JIdXVB2R+f8zAMBgNVHRMBAf8EAjAAMB8GA1Ud',
            'IwQYMBaAFErdBhYbvPZotXb1gba7Yhq6WoEvMCEGA1UdIAQaMBgwDAYKKwYBBAHW',
            'eQIFATAIBgZngQwBAgIwMAYDVR0fBCkwJzAloCOgIYYfaHR0cDovL3BraS5nb29n',
            'bGUuY29tL0dJQUcyLmNybDANBgkqhkiG9w0BAQsFAAOCAQEAfBoIl5qeaJ7NZ6hB',
            'WqeBZwbDV/DOHCPg3/84n8YGlfYdfXQpQdOWC5hfgEkkinBT0yp8dDTdXMUIT9Al',
            'ZMrxE54xJ1cU6FPuZPDWOnzV+6YEW6P9RnTbqKgYCNkHFiFwVvFRm5RTEGei5TLv',
            'l0zFDBusT/mgyvYBMIfW3vVPteEKKEz+aRCZHRiLAHbmJHj2+blVJeHGSF+eKN5q',
            'GWgk7/pMww4JAXsLQ0mmL8qdJKivuiNcyyhbr8IeERiVcItKqfBsX1nwyUnYFWY3',
            'HPkV+sXAPnpTGuxgYvTjcYDf8UO9lgDX5QubEFjjTuTIYAAabmc6Z4UKOS0O46Ne',
            'z28m7Q==',
            '-----END CERTIFICATE-----')
        # The SCT with the bad signature.
        sct_bytes = (
            '00ddeb1d2b7a0d4fa6208b81ad8168707e2e8e9d01d55c888d3d11c4cdb6ecbecc'
            '00000150421dfbb6000004030047304502200035de73784699d2ad8c3631aeda77'
            'f70b2c899492b16f051fd6d38d46afc892022100a4d1b58c63002e5d0862a9f623'
            'f67c8ccf5fc934bd28133fbc8f240aae4cab38'
        ).decode('hex')

        symantec_sct = client_pb2.SignedCertificateTimestamp()
        tls_message.decode(sct_bytes, symantec_sct)
        key_info = client_pb2.KeyInfo()
        key_info.type = client_pb2.KeyInfo.ECDSA
        key_info.pem_key = pem.to_pem(
            base64.decodestring(SYMANTEC_B64_KEY),
            'PUBLIC KEY')
        verifier = verify.LogVerifier(key_info)
        self.assertRaises(
            error.EncodingError,
            verifier.verify_sct,
            symantec_sct,
            [cert.Certificate.from_pem("\n".join(google_cert)),])


class CreateKeyInfoTest(unittest.TestCase):
    def test_create_key_info_with_rsa_key(self):
        key_info = verify.create_key_info_from_raw_key(
                base64.decodestring(VENAFI_B64_KEY))
        self.assertEqual(key_info.type, client_pb2.KeyInfo.RSA)
        self.assertTrue('PUBLIC KEY' in key_info.pem_key)

    def test_create_key_info_with_ecdsa_key(self):
        key_info = verify.create_key_info_from_raw_key(
                base64.decodestring(SYMANTEC_B64_KEY))
        self.assertEqual(key_info.type, client_pb2.KeyInfo.ECDSA)
        self.assertTrue('PUBLIC KEY' in key_info.pem_key)


if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    unittest.main()



from datetime import datetime
import gviz_api

class GvizGrapher(object):
    """A grapher that produces JSON output for the Google Visualizaton
    Datatable.
    """

    def __init__(self):
        pass

    def _get_nested_attr(self, item, attr):
        attributes = attr.split(".")
        ret = item
        for a in attributes:
            ret = getattr(ret, a)
        return ret

    def _convert_value(self, item, attr):
        """Convert the attribute of an item to a gviz value type."""
        value = self._get_nested_attr(item, attr[0])
        if len(attr) > 1 and attr[1] == "timestamp_ms":
            value = datetime.fromtimestamp(value/1000)
        return value

    def _convert_attr(self, attr):
        """Convert the attribute types accepted by us to ones recognised by
        gviz."""
        if len(attr) < 1 or attr[1] != "timestamp_ms":
            return attr
        tmp = list(attr)
        tmp[1] = "datetime"
        return tuple(tmp)

    def make_table(self, generator, columns, order_by=()):
        """A data table maker.
        Args:
           generator: A generator that yields the input data objects
           columns: A list of object attributes. Each attribute to be included
           as a column in the table must be represented as a tuple
           (attr [,data_type [,label [,custom_properties]]])
           Types supported by gviz are "string", "number", "boolean", "date",
           "datetime" or "timeofday".
           Additionally we've added "timestamp_ms" for millisecond since epoch,
           which we convert to "datetime".
        Returns:
           A JSON string containing the data
        """
        data = []
        for item in generator:
            row = [self._convert_value(item, attr) for attr in columns]
            data.append(row)

        data_table = gviz_api.DataTable(
            [self._convert_attr(attr) for attr in columns])
        data_table.LoadData(data)

        json = data_table.ToJSon(order_by=order_by)
        return json



#!/usr/bin/env python
"""This utility fetches the proof for a single certificate by its hash."""

import struct
import sys

from ct.client import log_client
from ct.crypto import cert
from ct.crypto import merkle
from ct.proto import client_pb2
from ct.serialization import tls_message
import gflags

FLAGS = gflags.FLAGS

gflags.DEFINE_string("cert", None, "Certificate file (PEM format) to fetch a "
                     "proof for.")
gflags.DEFINE_string("sct", None,
                     "SCT file (ProtoBuf/binary) of said certificate.")
gflags.DEFINE_bool("binary_sct", False, "SCT is in binary format")
gflags.DEFINE_integer("timestamp", None,
                     "Timestamp from SCT of said certificate.")
gflags.DEFINE_string("log_url", "https://ct.googleapis.com/pilot",
                     "URL of CT log.")
gflags.DEFINE_bool("verbose", False, "Verbose output or not.")


def create_leaf(timestamp, x509_cert_bytes):
    """Creates a MerkleTreeLeaf for the given X509 certificate."""
    leaf = client_pb2.MerkleTreeLeaf()
    leaf.version = client_pb2.V1
    leaf.leaf_type = client_pb2.TIMESTAMPED_ENTRY
    leaf.timestamped_entry.timestamp = timestamp
    leaf.timestamped_entry.entry_type = client_pb2.X509_ENTRY
    leaf.timestamped_entry.asn1_cert = x509_cert_bytes
    return tls_message.encode(leaf)

def construct_leaf_from_file(cert_file, cert_sct_timestamp):
    """Creates a MerkleTreeLeaf from a given PEM certificate file."""
    cert_to_lookup = cert.Certificate.from_pem_file(cert_file)
    return create_leaf(cert_sct_timestamp, cert_to_lookup.to_der())

def read_sct_from_file(sct_file):
    cert_sct = client_pb2.SignedCertificateTimestamp()
    cert_sct.ParseFromString(open(sct_file, 'rb').read())
    return cert_sct

def fetch_single_proof(leaf_hash, log_url):
    """Fetch the proof for the supplied certificate."""
    client = log_client.LogClient(log_url)
    sth = client.get_sth()
    if FLAGS.verbose:
      print "The log contains %d certificates." % (sth.tree_size)
      print "Tree root hash: %s" % (sth.sha256_root_hash.encode("hex"))

    proof_from_hash = client.get_proof_by_hash(
            leaf_hash, sth.tree_size)
    return sth, proof_from_hash

def run():
    """Fetch the proof for the supplied certificate."""
    #TODO(eranm): Attempt fetching the SCT for this chain if none was given.
    if FLAGS.sct:
        cert_sct = client_pb2.SignedCertificateTimestamp()
        sct_data = open(FLAGS.sct, 'rb').read()
        if FLAGS.binary_sct:
            tls_message.decode(sct_data, cert_sct)
        else:
            cert_sct.ParseFromString(sct_data)
        sct_timestamp = cert_sct.timestamp
        print 'SCT for cert:', cert_sct
    else:
        sct_timestamp = FLAGS.timestamp

    constructed_leaf = construct_leaf_from_file(FLAGS.cert, sct_timestamp)
    leaf_hash = merkle.TreeHasher().hash_leaf(constructed_leaf)
    if FLAGS.verbose:
      print "Leaf hash: %s" % (leaf_hash.encode("hex"))

    (sth, proof) = fetch_single_proof(leaf_hash, FLAGS.log_url);
    if FLAGS.verbose:
      print "Leaf index in tree is %d, proof has %d hashes" % (
          proof.leaf_index, len(proof.audit_path))
      print "Audit path: %s" % ([t.encode('hex') for t in proof.audit_path])

    verifier = merkle.MerkleVerifier()
    if verifier.verify_leaf_inclusion(constructed_leaf, proof.leaf_index,
                                      proof.audit_path, sth):
      print 'Proof verifies OK.'

if __name__ == '__main__':
    sys.argv = FLAGS(sys.argv)
    run()



#!/usr/bin/env python
import gflags
from google.protobuf import text_format
import logging
import os
import sys
import requests

from ct.client.db import sqlite_connection as sqlitecon
from ct.client import prober
from ct.client.db import sqlite_log_db
from ct.client.db import sqlite_temp_db
from ct.client.db import sqlite_cert_db
from ct.proto import client_pb2

FLAGS = gflags.FLAGS
gflags.DEFINE_string("ctlog_config", "ct/config/logs.config",
                     "Configuration file for log servers to monitor")
gflags.DEFINE_string("log_level", "WARNING", "logging level")
gflags.DEFINE_string("ct_sqlite_db", "/tmp/ct", "Location of the CT database")
gflags.DEFINE_string("ct_sqlite_temp_dir", "/tmp/ct_tmp", "Directory for "
                     "temporary CT data.")
gflags.DEFINE_string("ct_sqlite_cert_db", "/tmp/ct_cert", "Location of "
                     "certificate database.")
gflags.DEFINE_string("monitor_state_dir", "/tmp/ct_monitor",
                     "Filename prefix for monitor state. State for a given log "
                     "will be stored in a monitor_state_dir/log_id file")

def create_directory(directory):
    if not os.path.exists(directory):
        logging.info("Creating directory: %s" % directory)
        os.makedirs(directory)

if __name__ == '__main__':
    sys.argv = FLAGS(sys.argv)
    logging.basicConfig(level=FLAGS.log_level)

    create_directory(FLAGS.ct_sqlite_temp_dir)
    create_directory(FLAGS.monitor_state_dir)

    sqlite_log_db = sqlite_log_db.SQLiteLogDB(
        sqlitecon.SQLiteConnectionManager(FLAGS.ct_sqlite_db))

    sqlite_temp_db_factory = sqlite_temp_db.SQLiteTempDBFactory(
        sqlitecon.SQLiteConnectionManager(FLAGS.ct_sqlite_temp_dir + "/meta"),
                                          FLAGS.ct_sqlite_temp_dir)

    sqlite_cert_db = sqlite_cert_db.SQLiteCertDB(
            sqlitecon.SQLiteConnectionManager(FLAGS.ct_sqlite_cert_db))

    ctlogs = client_pb2.CtLogs()
    with open(FLAGS.ctlog_config, "r") as config:
        log_config = config.read()
    text_format.Merge(log_config, ctlogs)

    ct_server_list = []
    for log in ctlogs.ctlog:
        sqlite_log_db.update_log(log)
        ct_server_list.append(log.log_server)

    prober_thread = prober.ProberThread(ctlogs, sqlite_log_db,
                                        sqlite_cert_db,
                                        sqlite_temp_db_factory,
                                        FLAGS.monitor_state_dir)
    prober_thread.start()



'''
Generates a list of CT logs for use by OpenSSL.
It is in OpenSSL CONF format and the schema is documented here:
https://github.com/openssl/openssl/blob/master/doc/ssl/SSL_CTX_set_ctlog_list_file.pod
'''

def _log_id(log):
    # Use log URL as its ID because it should be unique and is probably
    # shorter and more readable in a comma-separated list than the log
    # description.
    return log["url"].replace(",", "")

def _openssl_list(items):
    '''
    Strip commas from any items used in a list in the OpenSSL CONF format,
    becayse they would be interpreted as delimiters.
    '''
    return ", ".join(x.replace(",", "") for x in items)

def _enabled_logs_conf(logs):
    return "enabled_logs = %s\n" % (
        _openssl_list(_log_id(log) for log in logs)
    )

def _log_conf(log, operators):
    operated_by = (operators[i]["name"] for i in log["operated_by"])

    return (
        "[%(id)s]\n"
        "description = %(description)s\n"
        "key = %(key)s\n"
        "url = https://%(url)s\n"
        "operated_by = %(operated_by)s\n" % {
            "id": _log_id(log),
            "description": log["description"],
            "key": log["key"],
            "url": log["url"],
            "operated_by": _openssl_list(operated_by),
    })

def generate_openssl_conf(json_log_list, output_path):
    '''Given a log list read from JSON, writes an OpenSSL log list to a file'''
    with open(output_path, "w") as output:
        logs = json_log_list["logs"]
        operators = json_log_list["operators"]
        log_confs = (_log_conf(log, operators) for log in logs)

        output.write(_enabled_logs_conf(logs) + "\n")
        output.write("\n".join(log_confs))



#!/usr/bin/env python

import unittest

from ct.client.db import sqlite_connection as sqlitecon
from ct.client.db import sqlite_log_db
from ct.client.db import log_db_test

class SQLiteLogDBTest(unittest.TestCase, log_db_test.LogDBTest):
    def setUp(self):
        self.database = sqlite_log_db.SQLiteLogDB(
            sqlitecon.SQLiteConnectionManager(":memory:", keepalive=True))
    def db(self):
        return self.database

if __name__ == '__main__':
    unittest.main()



"""Merkle trees.

Benchmark sample code:

>>> import os
>>> import timeit
>>> from ct.crypto import merkle
# generate a few blobs approx the size of a typical cert, takes a few seconds
>>> leaves = [os.urandom(2048) for i in xrange(65536)]
>>> hasher = merkle.TreeHasher()
>>> def timeav(code, n=20):
>>>     return timeit.timeit(
...         code, setup="from __main__ import hasher, leaves", number=n)/n
...

# time taken to hash 65536 certs individually
>>> print timeav("[hasher.hash_leaf(l) for l in leaves]")
1.14574944973

# time taken to hash 65536 certs in a full tree
>>> print timeav("hasher.hash_full_tree(leaves)")
1.50476324558
"""

import hashlib
import logging

from ct.crypto import error


def count_bits_set(i):
    # from https://wiki.python.org/moin/BitManipulation
    count = 0
    while i:
        i &= i - 1
        count += 1
    return count

def lowest_bit_set(i):
    # from https://wiki.python.org/moin/BitManipulation
    # but with 1-based indexing like in ffs(3) POSIX
    low = i & -i
    lowBit = 0
    while low:
        low >>= 1
        lowBit += 1
    return lowBit


class TreeHasher(object):
    """Merkle hasher with domain separation for leaves and nodes."""

    def __init__(self, hashfunc=hashlib.sha256):
        self.hashfunc = hashfunc

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, self.hashfunc)

    def __str__(self):
        return repr(self)

    def hash_empty(self):
        hasher = self.hashfunc()
        return hasher.digest()

    def hash_leaf(self, data):
        hasher = self.hashfunc()
        hasher.update("\x00" + data)
        return hasher.digest()

    def hash_children(self, left, right):
        hasher = self.hashfunc()
        hasher.update("\x01" + left + right)
        return hasher.digest()

    def _hash_full(self, leaves, l_idx, r_idx):
        """Hash the leaves between (l_idx, r_idx) as a valid entire tree.

        Note that this is only valid for certain combinations of indexes,
        depending on where the leaves are meant to be located in a parent tree.

        Returns:
            (root_hash, hashes): where root_hash is that of the entire tree,
            and hashes are that of the full (i.e. size 2^k) subtrees that form
            the entire tree, sorted in descending order of size.
        """
        width = r_idx - l_idx
        if width < 0 or l_idx < 0 or r_idx > len(leaves):
            raise IndexError("%s,%s not a valid range over [0,%s]" % (
                l_idx, r_idx, len(leaves)))
        elif width == 0:
            return self.hash_empty(), ()
        elif width == 1:
            leaf_hash = self.hash_leaf(leaves[l_idx])
            return leaf_hash, (leaf_hash,)
        else:
            # next smallest power of 2
            split_width = 2**((width - 1).bit_length() - 1)
            assert split_width < width <= 2*split_width
            l_root, l_hashes = self._hash_full(leaves, l_idx, l_idx+split_width)
            assert len(l_hashes) == 1 # left tree always full
            r_root, r_hashes = self._hash_full(leaves, l_idx+split_width, r_idx)
            root_hash = self.hash_children(l_root, r_root)
            return (root_hash, (root_hash,) if split_width*2 == width else
                                l_hashes + r_hashes)

    def hash_full_tree(self, leaves):
        """Hash a set of leaves representing a valid full tree."""
        root_hash, hashes = self._hash_full(leaves, 0, len(leaves))
        assert len(hashes) == count_bits_set(len(leaves))
        assert (self._hash_fold(hashes) == root_hash if hashes else
                root_hash == self.hash_empty())
        return root_hash

    def _hash_fold(self, hashes):
        rev_hashes = iter(hashes[::-1])
        accum = next(rev_hashes)
        for cur in rev_hashes:
            accum = self.hash_children(cur, accum)
        return accum


class CompactMerkleTree(object):
    """Compact representation of a Merkle Tree that permits only extension.

    Attributes:
        tree_size: Number of leaves in this tree.
        hashes: That of the full (i.e. size 2^k) subtrees that form this tree,
            sorted in descending order of size.
    """

    def __init__(self, hasher=TreeHasher(), tree_size=0, hashes=()):
        self.__hasher = hasher
        self._update(tree_size, hashes)

    def _update(self, tree_size, hashes):
        bits_set = count_bits_set(tree_size)
        num_hashes = len(hashes)
        if num_hashes != bits_set:
            msgfmt = "number of hashes != bits set in tree_size: %s vs %s"
            raise ValueError(msgfmt % (num_hashes, bits_set))
        self.__tree_size = tree_size
        self.__hashes = tuple(hashes)
        # height of the smallest subtree, or 0 if none exists (empty tree)
        self.__mintree_height = lowest_bit_set(tree_size)
        self.__root_hash = None

    def load(self, other):
        """Load this tree from a dumb data object for serialisation.

        The object must have attributes tree_size:int and hashes:list.
        """
        self._update(other.tree_size, other.hashes)

    def save(self, other):
        """Save this tree into a dumb data object for serialisation.

        The object must have attributes tree_size:int and hashes:list.
        """
        other.tree_size = self.__tree_size
        other.hashes[:] = self.__hashes

    def __copy__(self):
        return self.__class__(self.__hasher, self.__tree_size, self.__hashes)

    def __repr__(self):
        return "%s(%r, %r, %r)" % (
            self.__class__.__name__,
            self.__hasher, self.__tree_size, self.__hashes)

    def __len__(self):
        return self.__tree_size

    @property
    def tree_size(self):
        return self.__tree_size

    @property
    def hashes(self):
        return self.__hashes

    def root_hash(self):
        """Returns the root hash of this tree. (Only re-computed on change.)"""
        if self.__root_hash is None:
            self.__root_hash = (
                self.__hasher._hash_fold(self.__hashes)
                if self.__hashes else self.__hasher.hash_empty())
        return self.__root_hash

    def _push_subtree(self, leaves):
        """Extend with a full subtree <= the current minimum subtree.

        The leaves must form a full subtree, i.e. of size 2^k for some k. If
        there is a minimum subtree (i.e. __mintree_height > 0), then the input
        subtree must be smaller or of equal size to the minimum subtree.

        If the subtree is smaller (or no such minimum exists, in an empty tree),
        we can simply append its hash to self.hashes, since this maintains the
        invariant property of being sorted in descending size order.

        If the subtree is of equal size, we are in a similar situation to an
        addition carry. We handle it by combining the two subtrees into a larger
        subtree (of size 2^(k+1)), then recursively trying to add this new
        subtree back into the tree.

        Any collection of leaves larger than the minimum subtree must undergo
        additional partition to conform with the structure of a merkle tree,
        which is a more complex operation, performed by extend().
        """
        size = len(leaves)
        if count_bits_set(size) != 1:
            raise ValueError("invalid subtree with size != 2^k: %s" % size)
        # in general we want the highest bit, but here it's also the lowest bit
        # so just reuse that code instead of writing a new highest_bit_set()
        subtree_h, mintree_h = lowest_bit_set(size), self.__mintree_height
        if mintree_h > 0 and subtree_h > mintree_h:
            raise ValueError("subtree %s > current smallest subtree %s" % (
                subtree_h, mintree_h))
        root_hash, hashes = self.__hasher._hash_full(leaves, 0, size)
        assert hashes == (root_hash,)
        self.__push_subtree_hash(subtree_h, root_hash)

    def __push_subtree_hash(self, subtree_h, sub_hash):
        size, mintree_h = 1 << (subtree_h - 1), self.__mintree_height
        if subtree_h < mintree_h or mintree_h == 0:
            self._update(self.tree_size + size, self.hashes + (sub_hash,))
        else:
            assert subtree_h == mintree_h
            # addition carry - rewind the tree and re-try with bigger subtree
            prev_hash = self.hashes[-1]
            self._update(self.tree_size - size, self.hashes[:-1])
            new_mintree_h = self.__mintree_height
            assert mintree_h < new_mintree_h or new_mintree_h == 0
            next_hash = self.__hasher.hash_children(prev_hash, sub_hash)
            self.__push_subtree_hash(subtree_h + 1, next_hash)

    def append(self, new_leaf):
        """Append a new leaf onto the end of this tree."""
        self._push_subtree([new_leaf])

    def extend(self, new_leaves):
        """Extend this tree with new_leaves on the end.

        The algorithm works by using _push_subtree() as a primitive, calling
        it with the maximum number of allowed leaves until we can add the
        remaining leaves as a valid entire (non-full) subtree in one go.
        """
        size = len(new_leaves)
        final_size = self.tree_size + size
        idx = 0
        while True:
            # keep pushing subtrees until mintree_size > remaining
            max_h = self.__mintree_height
            max_size = 1 << (max_h - 1) if max_h > 0 else 0
            if max_h > 0 and size - idx >= max_size:
                self._push_subtree(new_leaves[idx:idx+max_size])
                idx += max_size
            else:
                break
        # fill in rest of tree in one go, now that we can
        if idx < size:
            root_hash, hashes = self.__hasher._hash_full(new_leaves, idx, size)
            self._update(final_size, self.hashes + hashes)
        assert self.tree_size == final_size

    def extended(self, new_leaves):
        """Returns a new tree equal to this tree extended with new_leaves."""
        new_tree = self.__copy__()
        new_tree.extend(new_leaves)
        return new_tree


class MerkleVerifier(object):
    """A utility class for doing Merkle path computations."""

    def __init__(self, hasher=TreeHasher()):
        self.hasher = hasher

    def __repr__(self):
        return "%r(hasher: %r)" % (self.__class__.__name__, self.hasher)

    def __str__(self):
        return "%s(hasher: %s)" % (self.__class__.__name__, self.hasher)

    @error.returns_true_or_raises
    def verify_tree_consistency(self, old_tree_size, new_tree_size, old_root,
                                new_root, proof):
        """Verify the consistency between two root hashes.

        old_tree_size must be <= new_tree_size.

        Args:
            old_tree_size: size of the older tree.
            new_tree_size: size of the newer_tree.
            old_root: the root hash of the older tree.
            new_root: the root hash of the newer tree.
            proof: the consistency proof.

        Returns:
            True. The return value is enforced by a decorator and need not be
                checked by the caller.

        Raises:
            ConsistencyError: the proof indicates an inconsistency
                (this is usually really serious!).
            ProofError: the proof is invalid.
            ValueError: supplied tree sizes are invalid.
        """
        old_size = long(old_tree_size)
        new_size = long(new_tree_size)

        if old_size < 0 or new_size < 0:
            raise ValueError("Negative tree size")

        if old_size > new_size:
            raise ValueError("Older tree has bigger size (%d vs %d), did "
                             "you supply inputs in the wrong order?" %
                             (old_size, new_size))

        if old_size == new_size:
            if old_root == new_root:
                if proof:
                    logging.warning("Trees are identical, ignoring proof")
                return True
            else:
                raise error.ConsistencyError("Inconsistency: different root "
                                             "hashes for the same tree size")

        if old_size == 0:
            if proof:
                # A consistency proof with an empty tree is an empty proof.
                # Anything is consistent with an empty tree, so ignore whatever
                # bogus proof was supplied. Note we do not verify here that the
                # root hash is a valid hash for an empty tree.
                logging.warning("Ignoring non-empty consistency proof for "
                                "empty tree.")
            return True

        # Now 0 < old_size < new_size
        # A consistency proof is essentially an audit proof for the node with
        # index old_size - 1 in the newer tree. The sole difference is that
        # the path is already hashed together into a single hash up until the
        # first audit node that occurs in the newer tree only.
        node = old_size - 1
        last_node = new_size - 1

        # While we are the right child, everything is in both trees, so move one
        # level up.
        while node % 2:
            node //= 2
            last_node //= 2

        p = iter(proof)
        try:
            if node:
                # Compute the two root hashes in parallel.
                new_hash = old_hash = p.next()
            else:
                # The old tree was balanced (2**k nodes), so we already have
                # the first root hash.
                new_hash = old_hash = old_root

            while node:
                if node % 2:
                    # node is a right child: left sibling exists in both trees.
                    next_node = p.next()
                    old_hash = self.hasher.hash_children(next_node, old_hash)
                    new_hash = self.hasher.hash_children(next_node, new_hash)
                elif node < last_node:
                    # node is a left child: right sibling only exists in the
                    # newer tree.
                    new_hash = self.hasher.hash_children(new_hash, p.next())
                # else node == last_node: node is a left child with no sibling
                # in either tree.
                node //= 2
                last_node //= 2

            # Now old_hash is the hash of the first subtree. If the two trees
            # have different height, continue the path until the new root.
            while last_node:
                new_hash = self.hasher.hash_children(new_hash, p.next())
                last_node //= 2

            # If the second hash does not match, the proof is invalid for the
            # given pair. If, on the other hand, the newer hash matches but the
            # older one doesn't, then the proof (together with the signatures
            # on the hashes) is proof of inconsistency.
            # Continue to find out.
            if new_hash != new_root:
                raise error.ProofError("Bad Merkle proof: second root hash "
                                       "does not match. Expected hash: %s "
                                       ", computed hash: %s" %
                                       (new_root.encode("base64").strip(),
                                        new_hash.encode("base64").strip()))
            elif old_hash != old_root:
                raise error.ConsistencyError("Inconsistency: first root hash "
                                             "does not match. Expected hash: "
                                             "%s, computed hash: %s" %
                                             (old_root.encode("base64").strip(),
                                              old_hash.encode("base64").strip())
                                             )

        except StopIteration:
            raise error.ProofError("Merkle proof is too short")

        # We've already verified consistency, so accept the proof even if
        # there's garbage left over (but log a warning).
        try:
            p.next()
        except StopIteration:
            pass
        else:
            logging.warning("Proof has extra nodes")
        return True

    def _calculate_root_hash_from_audit_path(self, leaf_hash, node_index,
                                             audit_path, tree_size):
        calculated_hash = leaf_hash
        last_node = tree_size - 1
        while last_node > 0:
            if not audit_path:
                raise error.ProofError('Proof too short: left with node index '
                                       '%d' % node_index)
            if node_index % 2:
                audit_hash = audit_path.pop(0)
                calculated_hash = self.hasher.hash_children(
                    audit_hash, calculated_hash)
            elif node_index < last_node:
                audit_hash = audit_path.pop(0)
                calculated_hash = self.hasher.hash_children(
                    calculated_hash, audit_hash)
            # node_index == last_node and node_index is even: A sibling does
            # not exist. Go further up the tree until node_index is odd so
            # calculated_hash will be used as the right-hand operand.
            node_index //= 2
            last_node //= 2
        if audit_path:
            raise error.ProofError('Proof too long: Left with %d hashes.' %
                                   len(audit_path))
        return calculated_hash

    @classmethod
    def audit_path_length(cls, index, tree_size):
        length = 0
        last_node = tree_size - 1
        while last_node > 0:
            if index % 2 or index < last_node:
                length += 1
            index //= 2
            last_node //= 2

        return length

    @error.returns_true_or_raises
    def verify_leaf_hash_inclusion(self, leaf_hash, leaf_index, proof, sth):
        """Verify a Merkle Audit Path.

        See section 2.1.1 of RFC6962 for the exact path description.

        Args:
            leaf_hash: The hash of the leaf for which the proof was provided.
            leaf_index: Index of the leaf in the tree.
            proof: A list of SHA-256 hashes representing the  Merkle audit path.
            sth: STH with the same tree size as the one used to fetch the proof.
            The sha256_root_hash from this STH will be compared against the
            root hash produced from the proof.

        Returns:
            True. The return value is enforced by a decorator and need not be
                checked by the caller.

        Raises:
            ProofError: the proof is invalid.
        """
        leaf_index = int(leaf_index)
        tree_size = int(sth.tree_size)
        #TODO(eranm): Verify signature over STH
        if tree_size <= leaf_index:
            raise ValueError("Provided STH is for a tree that is smaller "
                             "than the leaf index. Tree size: %d Leaf "
                             "index: %d" % (tree_size, leaf_index))
        if tree_size < 0 or leaf_index < 0:
            raise ValueError("Negative tree size or leaf index: "
                                   "Tree size: %d Leaf index: %d" %
                                   (tree_size, leaf_index))
        calculated_root_hash = self._calculate_root_hash_from_audit_path(
                leaf_hash, leaf_index, proof[:], tree_size)
        if calculated_root_hash == sth.sha256_root_hash:
            return True

        raise error.ProofError("Constructed root hash differs from provided "
                               "root hash. Constructed: %s Expected: %s" %
                               (calculated_root_hash.encode("base64").strip(),
                                sth.sha256_root_hash.encode("base64").strip()))

    @error.returns_true_or_raises
    def verify_leaf_inclusion(self, leaf, leaf_index, proof, sth):
        """Verify a Merkle Audit Path.

        See section 2.1.1 of RFC6962 for the exact path description.

        Args:
            leaf: The leaf for which the proof was provided.
            leaf_index: Index of the leaf in the tree.
            proof: A list of SHA-256 hashes representing the  Merkle audit path.
            sth: STH with the same tree size as the one used to fetch the proof.
            The sha256_root_hash from this STH will be compared against the
            root hash produced from the proof.

        Returns:
            True. The return value is enforced by a decorator and need not be
                checked by the caller.

        Raises:
            ProofError: the proof is invalid.
        """
        leaf_hash = self.hasher.hash_leaf(leaf)
        return self.verify_leaf_hash_inclusion(leaf_hash, leaf_index, proof,
                                               sth)



import gflags
import logging

from collections import defaultdict
from ct.client import reporter

FLAGS = gflags.FLAGS

class TextCertificateReport(reporter.CertificateReport):
    """Stores description of new entries between last verified STH and
    current."""

    def __init__(self):
        super(TextCertificateReport, self).__init__()

    def report(self):
        """Report stored changes and reset report."""
        super(TextCertificateReport, self).report()
        logging.info("Report:")
        logging.info("New entries since last verified STH: %s" %
                     self.new_entries_count)
        self.reset()

    def reset(self):
        """Clean up report.

        It's also ran at start."""
        self.new_entries_count = 0

    def _batch_scanned_callback(self, result):
        for _, log_index in result:
            msg = "Cert %d" % log_index
            self.new_entries_count += 1



import abc
from ct.client.db import log_db
from ct.client.db import database
from ct.proto import client_pb2

# This class provides common tests for all CT log database implementations.
# It only inherits from object so that unittest won't attempt to run the test_*
# methods on this class. Derived classes should use multiple inheritance
# from LogDBTest and unittest.TestCase to get test automation.
class LogDBTest(object):
    """All LogDB tests should derive from this class as well as
    unittest.TestCase."""
    __metaclass__ = abc.ABCMeta

    # Set up a default fake test log server and STH.
    default_log = client_pb2.CtLogMetadata()
    default_log.log_server = "test"
    default_log.log_id = "c29tZWtleWlk"  # b64("somekeyid")
    default_log.public_key_info.type = client_pb2.KeyInfo.ECDSA
    default_log.public_key_info.pem_key = "base64encodedkey"

    default_sth = client_pb2.AuditedSth()
    default_sth.sth.timestamp = 1234
    default_sth.sth.sha256_root_hash = "base64hash"
    default_sth.audit.status = client_pb2.VERIFIED

    @abc.abstractmethod
    def db(self):
        """Derived classes must override to initialize a database."""
        pass

    def test_add_log(self):
        self.db().add_log(LogDBTest.default_log)
        generator = self.db().logs()
        metadata = generator.next()
        self.assertEqual(metadata, LogDBTest.default_log)
        self.assertRaises(StopIteration, generator.next)

    def test_update_log(self):
        self.db().add_log(LogDBTest.default_log)
        self.db().store_sth(LogDBTest.default_log.log_server,
                            LogDBTest.default_sth)

        new_log = client_pb2.CtLogMetadata()
        new_log.CopyFrom(LogDBTest.default_log)
        new_log.public_key_info.pem_key = "newkey"
        self.db().update_log(new_log)
        generator = self.db().logs()
        metadata = generator.next()
        self.assertEqual(metadata, new_log)
        self.assertRaises(StopIteration, generator.next)

        # Should still be able to access STHs after updating log metadata
        read_sth = self.db().get_latest_sth(new_log.log_server)
        self.assertTrue(read_sth)
        self.assertEqual(LogDBTest.default_sth, read_sth)

    def test_update_log_adds_log(self):
        self.db().update_log(LogDBTest.default_log)
        generator = self.db().logs()
        metadata = generator.next()
        self.assertEqual(metadata, LogDBTest.default_log)
        self.assertRaises(StopIteration, generator.next)

    def test_store_sth(self):
        self.db().add_log(LogDBTest.default_log)
        self.db().store_sth(LogDBTest.default_log.log_server,
                            LogDBTest.default_sth)
        read_sth = self.db().get_latest_sth(LogDBTest.default_log.log_server)
        self.assertTrue(read_sth)
        self.assertEqual(LogDBTest.default_sth, read_sth)

    def test_store_sth_ignores_duplicate(self):
        self.db().add_log(LogDBTest.default_log)
        self.db().store_sth(LogDBTest.default_log.log_server,
                            LogDBTest.default_sth)
        duplicate_sth = client_pb2.AuditedSth()
        duplicate_sth.audit.status = client_pb2.VERIFY_ERROR
        self.db().store_sth(LogDBTest.default_log.log_server, duplicate_sth)
        read_sth = self.db().get_latest_sth(LogDBTest.default_log.log_server)
        self.assertTrue(read_sth)
        self.assertEqual(LogDBTest.default_sth, read_sth)

    def test_log_not_found_raises(self):
        self.assertRaises(database.KeyError, self.db().store_sth,
                          LogDBTest.default_log.log_server,
                          LogDBTest.default_sth)

    def test_get_latest_sth_returns_latest(self):
        self.db().add_log(LogDBTest.default_log)
        self.db().store_sth(LogDBTest.default_log.log_server,
                            LogDBTest.default_sth)
        new_sth = client_pb2.AuditedSth()
        new_sth.CopyFrom(LogDBTest.default_sth)
        new_sth.sth.timestamp = LogDBTest.default_sth.sth.timestamp - 1
        self.db().store_sth(LogDBTest.default_log.log_server, new_sth)
        read_sth = self.db().get_latest_sth(LogDBTest.default_log.log_server)
        self.assertIsNotNone(read_sth)
        self.assertEqual(LogDBTest.default_sth, read_sth)

    def test_get_latest_sth_returns_none_if_empty(self):
        self.db().add_log(LogDBTest.default_log)
        self.assertIsNone(self.db().get_latest_sth(
            LogDBTest.default_log.log_server))

    def test_get_latest_sth_honours_log_server(self):
        self.db().add_log(LogDBTest.default_log)
        self.db().store_sth(LogDBTest.default_log.log_server,
                            LogDBTest.default_sth)
        new_sth = client_pb2.AuditedSth()
        new_sth.CopyFrom(LogDBTest.default_sth)
        new_sth.sth.timestamp = LogDBTest.default_sth.sth.timestamp + 1

        new_log = client_pb2.CtLogMetadata()
        new_log.log_server = "test2"
        new_log.log_id = "c29tZW90aGVya2V5aWQ="  # b64("someotherkeyid")
        self.db().add_log(new_log)

        new_sth.sth.sha256_root_hash = "hash2"
        self.db().store_sth(new_log.log_server, new_sth)
        read_sth = self.db().get_latest_sth(LogDBTest.default_log.log_server)
        self.assertIsNotNone(read_sth)
        self.assertEqual(LogDBTest.default_sth, read_sth)

    def test_scan_latest_sth_range_finds_all(self):
        self.db().add_log(LogDBTest.default_log)
        for i in range(4):
            sth = client_pb2.AuditedSth()
            sth.sth.timestamp = i
            sth.sth.sha256_root_hash = "hash-%d" % i
            self.db().store_sth(LogDBTest.default_log.log_server, sth)

        generator = self.db().scan_latest_sth_range(
            LogDBTest.default_log.log_server)
        for i in range(3, -1, -1):
            sth = generator.next()
            # Scan runs in descending timestamp order
            self.assertEqual(sth.sth.timestamp, i)
            self.assertEqual(sth.sth.sha256_root_hash, "hash-%d" % i)

        self.assertRaises(StopIteration, generator.next)

    def test_scan_latest_sth_range_honours_log_server(self):
        for i in range(4):
            log = client_pb2.CtLogMetadata()
            log.log_server = "test-%d" % i
            self.db().add_log(log)
        for i in range(4):
            sth = client_pb2.AuditedSth()
            sth.sth.timestamp = i
            sth.sth.sha256_root_hash = "hash-%d" % i
            self.db().store_sth("test-%d" % i, sth)

        for i in range(4):
            generator = self.db().scan_latest_sth_range("test-%d" % i)
            sth = generator.next()
            self.assertEqual(sth.sth.timestamp, i)
            self.assertEqual(sth.sth.sha256_root_hash, "hash-%d" % i)

    def test_scan_latest_sth_range_honours_range(self):
        self.db().add_log(LogDBTest.default_log)
        for i in range(4):
            sth = client_pb2.AuditedSth()
            sth.sth.timestamp = i
            sth.sth.sha256_root_hash = "hash-%d" % i
            self.db().store_sth(LogDBTest.default_log.log_server, sth)

        generator = self.db().scan_latest_sth_range("test", start=1, end=2)
        for i in range(2):
            sth = generator.next()
            self.assertEqual(sth.sth.timestamp, 2-i)
            self.assertEqual(sth.sth.sha256_root_hash, "hash-%d" % (2-i))

        self.assertRaises(StopIteration, generator.next)

    def test_scan_latest_sth_range_honours_limit(self):
        self.db().add_log(LogDBTest.default_log)
        for i in range(4):
            sth = client_pb2.AuditedSth()
            sth.sth.timestamp = i
            sth.sth.sha256_root_hash = "hash-%d" % i
            self.db().store_sth(LogDBTest.default_log.log_server, sth)

        generator = self.db().scan_latest_sth_range("test", limit=1)
        sth = generator.next()
        # Returns most recent
        self.assertEqual(sth.sth.timestamp, 3)
        self.assertEqual(sth.sth.sha256_root_hash, "hash-%d" % 3)

        self.assertRaises(StopIteration, generator.next)



#!/usr/bin/env python

import unittest

import base64
import json
import mock
import requests
import sys

from ct.client import log_client
from ct.client import log_client_test_util as test_util
from ct.crypto import merkle
from ct.proto import client_pb2
import gflags

FLAGS = gflags.FLAGS


class LogClientTest(unittest.TestCase):
    class FakeHandler(test_util.FakeHandlerBase):

        # A class that mimics requests.models.Response
        class FakeResponse(object):
            def __init__(self, code, reason, json_content=None):
                self.status_code = code
                self.reason = reason
                self.headers = ''
                if json_content is not None:
                    self.content = json.dumps(json_content)
                else:
                    self.content = ""

        @classmethod
        def make_response(cls, code, reason, json_content=None):
            return cls.FakeResponse(code, reason, json_content=json_content)

    @staticmethod
    def one_shot_client(json_content):
        """Make a one-shot client and give it a mock response."""
        mock_handler = mock.Mock()
        mock_handler.get_response_body.return_value = json.dumps(json_content)
        return log_client.LogClient("some address", handler=mock_handler)

    def default_client(self):
        # A client whose responder is configured to answer queries for the
        # correct uri.
        return log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI))

    def test_get_sth(self):
        client = self.default_client()
        sth_response = client.get_sth()

        self.assertEqual(sth_response.timestamp,
                         test_util.DEFAULT_STH.timestamp)
        self.assertEqual(sth_response.tree_size,
                         test_util.DEFAULT_STH.tree_size)
        self.assertEqual(sth_response.sha256_root_hash,
                         test_util.DEFAULT_STH.sha256_root_hash)
        self.assertEqual(sth_response.tree_head_signature,
                         test_util.DEFAULT_STH.tree_head_signature)

    def test_get_sth_raises_on_invalid_response(self):
        json_sth = test_util.sth_to_json(test_util.DEFAULT_STH)
        json_sth.pop("timestamp")
        client = self.one_shot_client(json_sth)
        self.assertRaises(log_client.InvalidResponseError, client.get_sth)

    def test_get_sth_raises_on_invalid_base64(self):
        json_sth = test_util.sth_to_json(test_util.DEFAULT_STH)
        json_sth["tree_head_signature"] = "garbagebase64^^^"
        client = self.one_shot_client(json_sth)
        self.assertRaises(log_client.InvalidResponseError, client.get_sth)

    def test_get_entries(self):
        client = self.default_client()
        returned_entries = list(client.get_entries(0, 9))
        self.assertTrue(test_util.verify_entries(returned_entries, 0, 9))

    def test_get_entries_raises_on_invalid_response(self):
        json_entries = test_util.entries_to_json(test_util.make_entries(4, 4))
        json_entries["entries"][0].pop("leaf_input")

        client = self.one_shot_client(json_entries)
        entries = client.get_entries(4, 4)
        self.assertRaises(log_client.InvalidResponseError,
                          entries.next)

    def test_get_entries_raises_immediately_on_invalid_base64(self):
        json_entries = test_util.entries_to_json(test_util.make_entries(3, 4))
        json_entries["entries"][1]["leaf_input"] = "garbagebase64^^^"

        client = self.one_shot_client(json_entries)
        entries = client.get_entries(3, 4)
        # We shouldn't see anything, even if the first entry appeared valid.
        self.assertRaises(log_client.InvalidResponseError,
                          entries.next)

    def test_get_entries_raises_on_empty_response(self):
        empty_entries = test_util.entries_to_json([])
        client = self.one_shot_client(empty_entries)

        entries = client.get_entries(4, 4)
        self.assertRaises(log_client.InvalidResponseError,
                          entries.next)

    def test_get_entries_raises_on_too_large_response(self):
        large_response = test_util.entries_to_json(
            test_util.make_entries(4, 5))

        client = self.one_shot_client(large_response)
        entries = client.get_entries(4, 4)
        self.assertRaises(log_client.InvalidResponseError,
                          entries.next)

    def test_get_entries_returns_all_in_batches(self):
        mock_handler = mock.Mock()
        fake_responder = self.FakeHandler(test_util.DEFAULT_URI)
        mock_handler.get_response_body.side_effect = (
            fake_responder.get_response_body)

        client = log_client.LogClient(test_util.DEFAULT_URI,
                                      handler=mock_handler)
        returned_entries = list(client.get_entries(0, 9, batch_size=4))
        self.assertTrue(test_util.verify_entries(returned_entries, 0, 9))
        self.assertEqual(3, len(mock_handler.get_response_body.call_args_list))

        # Same as above, but using a flag to control the batch size.
        mock_handler.reset_mock()
        # TODO(ekasper): find a more elegant and robust way to save flags.
        original = FLAGS.entry_fetch_batch_size
        FLAGS.entry_fetch_batch_size = 4
        returned_entries = list(client.get_entries(0, 9))
        FLAGS.entry_fetch_batch_size = original
        self.assertTrue(test_util.verify_entries(returned_entries, 0, 9))
        self.assertEqual(3, len(mock_handler.get_response_body.call_args_list))

    def test_get_entries_returns_all_for_limiting_server(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, entry_limit=3))
        returned_entries = list(client.get_entries(0, 9))
        self.assertTrue(test_util.verify_entries(returned_entries, 0, 9))

    def test_get_entries_returns_partial_if_log_returns_partial(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, tree_size=3))
        entries = client.get_entries(0, 9)
        partial = []
        for _ in range(3):
            partial.append(entries.next())
        self.assertTrue(test_util.verify_entries(partial, 0, 2))
        self.assertRaises(log_client.HTTPClientError, entries.next)

    def test_get_sth_consistency(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, tree_size=3))
        proof = client.get_sth_consistency(1, 2)
        self.assertEqual(proof, test_util.DEFAULT_FAKE_PROOF)

    def test_get_sth_consistency_trivial(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, tree_size=3))
        self.assertEqual(client.get_sth_consistency(0, 0), [])
        self.assertEqual(client.get_sth_consistency(0, 2), [])
        self.assertEqual(client.get_sth_consistency(2, 2), [])

    def test_get_sth_consistency_raises_on_invalid_input(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, tree_size=3))
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_sth_consistency, -1, 1)
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_sth_consistency, -3, -1)
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_sth_consistency, 3, 1)

    def test_get_sth_consistency_raises_on_client_error(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, tree_size=3))
        self.assertRaises(log_client.HTTPClientError,
                          client.get_sth_consistency, 1, 5)

    def test_get_sth_consistency_raises_on_invalid_response(self):
        client = self.one_shot_client({})
        self.assertRaises(log_client.InvalidResponseError,
                          client.get_sth_consistency, 1, 2)

    def test_get_sth_consistency_raises_on_invalid_base64(self):
        json_proof = {"consistency": ["garbagebase64^^^"]}
        client = self.one_shot_client(json_proof)
        self.assertRaises(log_client.InvalidResponseError,
                          client.get_sth_consistency, 1, 2)

    def test_get_roots(self):
        client = self.default_client()
        roots = client.get_roots()
        self.assertEqual(roots, test_util.DEFAULT_FAKE_ROOTS)

    def test_get_roots_raises_on_invalid_response(self):
        client = self.one_shot_client({})
        self.assertRaises(log_client.InvalidResponseError, client.get_roots)

    def test_get_roots_raises_on_invalid_base64(self):
        json_roots = {"certificates": ["garbagebase64^^^"]}
        client = self.one_shot_client(json_roots)
        self.assertRaises(log_client.InvalidResponseError, client.get_roots)

    def test_get_entry_and_proof(self):
        client = self.default_client()
        entry_and_proof = client.get_entry_and_proof(1, 2)
        self.assertEqual(entry_and_proof.entry, test_util.make_entry(1))
        self.assertEqual(entry_and_proof.audit_path,
                         test_util.DEFAULT_FAKE_PROOF)

    def test_get_entry_and_proof_raises_on_invalid_input(self):
        client = self.default_client()
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_entry_and_proof, -1, 1)
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_entry_and_proof, -3, -1)
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_entry_and_proof, 3, 1)

    def test_get_entry_and_proof_raises_on_client_error(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, tree_size=3))
        self.assertRaises(log_client.HTTPClientError,
                          client.get_entry_and_proof, 1, 5)

    def test_get_entry_and_proof_raises_on_invalid_response(self):
        json_response = test_util.entry_and_proof_to_json(
            test_util.make_entry(1), test_util.DEFAULT_FAKE_PROOF)
        json_response.pop("leaf_input")
        client = self.one_shot_client(json_response)
        self.assertRaises(log_client.InvalidResponseError,
                          client.get_entry_and_proof, 1, 2)

    def test_get_entry_and_proof_raises_on_invalid_base64(self):
        json_response = test_util.entry_and_proof_to_json(
            test_util.make_entry(1), test_util.DEFAULT_FAKE_PROOF)
        json_response["leaf_input"] = ["garbagebase64^^^"]
        client = self.one_shot_client(json_response)
        self.assertRaises(log_client.InvalidResponseError,
                          client.get_entry_and_proof, 1, 2)

    def test_get_proof_by_hash(self):
        client = self.default_client()
        entry = test_util.make_entry(1)
        hasher = merkle.TreeHasher()
        leaf_hash = hasher.hash_leaf(entry.leaf_input)

        proof_by_hash = client.get_proof_by_hash(leaf_hash, 2)
        self.assertEqual(proof_by_hash.audit_path, test_util.DEFAULT_FAKE_PROOF)
        self.assertEqual(proof_by_hash.leaf_index, 1)

    def test_get_proof_by_hash_raises_on_invalid_input(self):
        client = self.default_client()
        leaf_hash = "hash"
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_proof_by_hash, leaf_hash, 0)
        self.assertRaises(log_client.InvalidRequestError,
                          client.get_proof_by_hash, leaf_hash, -1)

    def test_get_proof_by_hash_raises_on_unknown_hash(self):
        client = log_client.LogClient(test_util.DEFAULT_URI, self.FakeHandler(
            test_util.DEFAULT_URI, tree_size=3))
        leaf_hash = "bogus"
        self.assertRaises(log_client.HTTPClientError,
                          client.get_proof_by_hash, leaf_hash, 2)

    def test_get_proof_by_hash_raises_on_invalid_response(self):
        json_response = test_util.proof_and_index_to_json(
            test_util.DEFAULT_FAKE_PROOF, 1)
        json_response.pop("leaf_index")
        client = self.one_shot_client(json_response)
        self.assertRaises(log_client.InvalidResponseError,
                          client.get_proof_by_hash, "hash", 2)

    def test_get_proof_by_hash_raises_on_invalid_base64(self):
        json_response = test_util.proof_and_index_to_json(
            test_util.DEFAULT_FAKE_PROOF, 1)
        json_response["leaf_index"] = "garbagebase64^^^"
        client = self.one_shot_client(json_response)
        self.assertRaises(log_client.InvalidResponseError,
                          client.get_proof_by_hash, "hash", 2)

    def _verify_sct_contents(self, sct):
        LOG_ID = base64.b64decode(
            'pLkJkLQYWBSHuxOizGdwCjw1mAT5G9+443fNDsgN3BA=')
        self.assertEqual(client_pb2.V1, sct.version)
        self.assertEqual(LOG_ID, sct.id.key_id)
        self.assertEqual(1373015623951L, sct.timestamp)
        self.assertEqual(client_pb2.DigitallySigned.SHA256,
                         sct.signature.hash_algorithm)
        self.assertEqual(client_pb2.DigitallySigned.ECDSA,
                         sct.signature.sig_algorithm)
        RAW_SIGNATURE = ('304402202080fb4a50c159e3398d9cf85cec0b3b551d4379db1d'
                         '820b3d6bca52107a32180220286b2f20f0d98039f14e3198f1f4'
                         '81c24975e5d0344d4a96e5ec761c253bc84f').decode('hex')
        self.assertEqual(RAW_SIGNATURE, sct.signature.signature)

    def test_add_valid_chain(self):
        certs_chain = ["one", "two", "three"]
        json_sct_response = (
            '{"sct_version":0,"id":"pLkJkLQYWBSHuxOizGdwCjw1m'
            'AT5G9+443fNDsgN3BA=","timestamp":1373015623951,\n'
            '"extensions":"",\n'
            '"signature":"BAMARjBEAiAggPtKUMFZ4zmNnPhc7As7VR1Dedsdggs9a8pSEHoy'
            'GAIgKGsvIPDZgDnxTjGY8fSBwkl15dA0TUqW5ex2HCU7yE8="}')
        mock_handler = mock.Mock()
        mock_handler.post_response_body.return_value = json_sct_response
        client = log_client.LogClient("http://ctlog", handler=mock_handler)
        received_sct = client.add_chain(certs_chain)
        mock_handler.post_response_body.assert_called_once_with(
            "http://ctlog/ct/v1/add-chain",
            post_data={'chain': [base64.b64encode(t) for t in certs_chain]})
        self._verify_sct_contents(received_sct)

    def test_fails_parsing_sct_invalid_version(self):
        json_sct_response = (
            '{"sct_version":2,"id":"pLkJkLQYWBSHuxOizGdwCjw1m'
            'AT5G9+443fNDsgN3BA=","timestamp":1373015623951,\n'
            '"extensions":"",\n'
            '"signature":"BAMARjBEAiAggPtKUMFZ4zmNnPhc7As7VR1Dedsdggs9a8pSEHoy'
            'GAIgKGsvIPDZgDnxTjGY8fSBwkl15dA0TUqW5ex2HCU7yE8="}')
        client = self.default_client()
        self.assertRaises(log_client.InvalidResponseError,
                          client._parse_sct, json_sct_response)

    def test_fails_parsing_sct_missing_contents(self):
        json_sct_response = (
            '{"sct_version":0,"id":"pLkJkLQYWBSHuxOizGdwCjw1m'
            'AT5G9+443fNDsgN3BA=",\n'
            '"extensions":"",\n'
            '"signature":"BAMARjBEAiAggPtKUMFZ4zmNnPhc7As7VR1Dedsdggs9a8pSEHoy'
            'GAIgKGsvIPDZgDnxTjGY8fSBwkl15dA0TUqW5ex2HCU7yE8="}')
        client = self.default_client()
        self.assertRaises(log_client.InvalidResponseError,
                          client._parse_sct, json_sct_response)

class RequestHandlerTest(unittest.TestCase):
    class RequestSideEffect:
        def __init__(self, num_failures, canned_response=None):
            self._num_failures = num_failures
            self._canned_response = canned_response

        def __call__(self, req_url, params, timeout, verify):
            if self._num_failures <= 0:
                return self._canned_response
            self._num_failures = self._num_failures - 1
            raise requests.exceptions.ConnectionError("incomplete read!")

    def test_uri_with_params(self):
        self.assertEqual(
            'http://www.google.com',
            log_client.RequestHandler._uri_with_params('http://www.google.com',
                                                      {}))

        self.assertIn(
            log_client.RequestHandler._uri_with_params('http://www.google.com',
                                                       {'a': 1, 'b': 2}),
            ['http://www.google.com?a=1&b=2', 'http://www.google.com?b=2&a=1'])

        self.assertIn(
            log_client.RequestHandler._uri_with_params('http://www.google.com/',
                                                       {'a': 1,
                                                        'b': 'foo bar'}),
            ['http://www.google.com/?a=1&b=foo+bar',
             'http://www.google.com/?b=foo+bar&a=1'])

    def test_get_response_one_retry(self):
        expected_body = 'valid_body'
        handler = log_client.RequestHandler(num_retries=1)
        canned_response = requests.models.Response()
        canned_response.status_code = 200
        canned_response._content = expected_body
        log_client.requests.get = mock.Mock(
                side_effect=
                self.RequestSideEffect(1, canned_response))

        received_response = handler.get_response('http://www.example.com')
        self.assertEqual(expected_body, received_response.content)
        self.assertEqual(200, received_response.status_code)

    def test_get_response_too_many_retries(self):
        handler = log_client.RequestHandler(num_retries=3)
        canned_response = requests.models.Response()
        canned_response.status_code = 200
        canned_response._content = 'body'
        log_client.requests.get = mock.Mock(
                side_effect=self.RequestSideEffect(
                    4, canned_response))

        self.assertRaises(log_client.HTTPError,
                          handler.get_response, ('http://www.example.com'))


if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    unittest.main()



#!/usr/bin/env python

import unittest
import sqlite3

from ct.client.db import sqlite_connection as sqlitecon

class SQLiteConnectionTest(unittest.TestCase):
    def test_connection_works(self):
        with sqlitecon.SQLiteConnection(":memory:") as conn:
            conn.execute("CREATE TABLE words(word TEXT)")
            conn.execute("INSERT INTO words VALUES (?)", ("hello",))
            results = conn.execute("SELECT * FROM words")
            self.assertEqual("hello", results.next()["word"])
            self.assertRaises(StopIteration, results.next)

    def test_exit_autocommits(self):
        # Need keepalive=True as the memory db only lives as long as the
        # connection.
        with sqlitecon.SQLiteConnection(":memory:", keepalive=True) as conn:
            conn.execute("CREATE TABLE words(word TEXT)")
            conn.execute("INSERT INTO words VALUES (?)", ("hello",))
        results = conn.execute("SELECT * FROM words")
        self.assertEqual("hello", results.next()["word"])

    def test_no_keepalive_closes_connection(self):
        with sqlitecon.SQLiteConnection(":memory:", keepalive=False) as conn:
            conn.execute("CREATE TABLE words(word TEXT)")
        self.assertRaises(sqlite3.ProgrammingError, conn.execute,
                          "SELECT * FROM words")

class SQLiteConnectionManagerTest(unittest.TestCase):
    def test_get_connection(self):
        mgr = sqlitecon.SQLiteConnectionManager(":memory:")
        self.assertIsInstance(mgr.get_connection(), sqlitecon.SQLiteConnection)

    def test_keepalive_returns_same_connection(self):
        mgr = sqlitecon.SQLiteConnectionManager(":memory:", keepalive=True)
        with mgr.get_connection() as conn:
            conn.execute("CREATE TABLE words(word TEXT)")
            conn.execute("INSERT INTO words VALUES (?)", ("hello",))

        with mgr.get_connection() as conn:
            results = conn.execute("SELECT * FROM words")
            self.assertEqual("hello", results.next()["word"])

    def test_no_keepalive_returns_new_connection(self):
        mgr = sqlitecon.SQLiteConnectionManager(":memory:", keepalive=False)
        with mgr.get_connection() as conn:
            conn.execute("CREATE TABLE words(word TEXT)")
            conn.execute("INSERT INTO words VALUES (?)", ("hello",))

        with mgr.get_connection() as conn:
            self.assertRaises(sqlite3.OperationalError, conn.execute,
                              "SELECT * FROM words")

if __name__ == '__main__':
    unittest.main()



import gflags
import logging
import threading
import time

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("probe_frequency_secs", 10*60,
                      "How often to probe the logs for updates")

from ct.client import async_log_client
from ct.client import monitor
from ct.client import state
from ct.crypto import merkle
from ct.crypto import verify
from twisted.internet import reactor
from twisted.web import client as twisted_client

class ProberThread(threading.Thread):
    """A prober for scheduled updating of the log view."""
    def __init__(self, ct_logs, db, cert_db, temp_db_factory, monitor_state_dir,
                 agent=None, state_keeper_class=None):
        """Initialize from a CtLogs proto."""
        threading.Thread.__init__(self)

        self.__monitors = []
        self.__db = db
        if not agent:
            agent = twisted_client.Agent(reactor)
        if not state_keeper_class:
            state_keeper_class = state.StateKeeper

        for log in ct_logs.ctlog:
            if not log.log_server or not log.log_id or not log.public_key_info:
                raise RuntimeError("Cannot start monitor: log proto has "
                                   "missing or empty fields: %s" % log)

            temp_db = temp_db_factory.create_storage(log.log_server)
            client = async_log_client.AsyncLogClient(agent,
                                               log.log_server,
                                               temp_db)
            hasher = merkle.TreeHasher()
            verifier = verify.LogVerifier(log.public_key_info,
                                          merkle.MerkleVerifier(hasher))
            # Convert from standard Base64 to URL-safe Base64 so that the log ID
            # can be used as part of a file path.
            log_id_urlsafe = log.log_id.replace('/', '_').replace('+', '-')
            state_keeper = state_keeper_class(monitor_state_dir +
                                             "/" + log_id_urlsafe)
            log_key = db.get_log_id(log.log_server)
            self.__monitors.append(monitor.Monitor(client, verifier, hasher, db,
                                                   cert_db, log_key,
                                                   state_keeper))

        self.__last_update_start_time = 0
        self.__stopped = False
        self.__called_later = None

    def __repr__(self):
       return "%r(%r)" % (self.__class__.__name__, self.__monitors)

    def __str__(self):
       return "%s(%s)" % (self.__class__.__name__, self.__monitors)

    def _log_probed_callback(self, success, monitor):
        if success:
            logging.info("Data for %s updated: latest timestamp is %s" %
                         (monitor.servername,
                          time.strftime("%c", time.localtime(
                            monitor.data_timestamp/1000))))
        else:
            logging.error("Failed to update data for %s: latest timestamp "
                          "is %s" % (monitor.servername,
                                     time.strftime("%c", time.localtime(
                            monitor.data_timestamp/1000))))
        self.__probed += 1
        if self.__probed == len(self.__monitors):
            self._all_logs_probed()

    def _all_logs_probed(self):
        logging.info("Probe loop completed in %d seconds" %
                     (time.time() - self.__start_time))
        sleep_time = max(0, self.__start_time +
                         FLAGS.probe_frequency_secs - time.time())
        logging.info("Next probe loop in: %d seconds" % sleep_time)
        self.__called_later = reactor.callLater(sleep_time,
                                                self.probe_all_logs)


    def _has_outstanding_call_later(self):
        return self.__called_later and self.__called_later.active()

    def probe_all_logs(self):
        logging.info("Starting probe loop...")
        self.__called_later = None
        self.__start_time = time.time()
        self.__probed = 0
        """Loop through all logs in the list and check for updates."""
        if self.__monitors:
            for monitor_ in self.__monitors:
                monitor_result = monitor_.update()
                monitor_result.addCallback(self._log_probed_callback, monitor_)
                # TODO(hadfieldp): do we need an errback too?
        else:
            # If we're configured with no monitors we still need to behave
            # correctly in order for the reactor to be stoppable.
            self._all_logs_probed()
        logging.info("Done starting probe loop.")

    def run(self):
        logging.info("Running reactor...")
        self.__called_later = reactor.callLater(0, self.probe_all_logs)
        reactor.run(installSignalHandlers=0)
        logging.info("Reactor no longer running.")

    def stop(self):
        logging.info("Stopping reactor...")
        if self._has_outstanding_call_later():
            self.__called_later.cancel()
            self.__called_later = None
        reactor.stop()
        logging.info("Reactor stopped.")



#!/usr/bin/env python
"""Extracts all Precertificates from the log."""

import os
import sys

import gflags

from ct.client import scanner
from ct.proto import client_pb2

FLAGS = gflags.FLAGS

gflags.DEFINE_integer("multi", 2, "Number of cert fetching and parsing "
                      "processes to use, in addition to the main process.")

gflags.DEFINE_string("output_directory", None,
                     "Output directory for individual Precertificates.")

gflags.DEFINE_integer("start_entry", 0, "Log entry to start from.")



def _precert_matches(certificate, entry_type, extra_data, certificate_index):
    """Matcher function for the scanner. Returns a filename and certificate in
     PEM format if it's a precertificate, None otherwise."""
    if entry_type == client_pb2.PRECERT_ENTRY:
        return ("precert_%d.pem" % certificate_index, certificate.to_pem())
    return None


def write_matched_certificate(matcher_output):
    output_file, der_data = matcher_output
    with open(os.path.join(FLAGS.output_directory, output_file), "wb") as f:
        f.write(der_data)


def run():
    if not FLAGS.output_directory:
        raise Exception("Certificates output directory must be specified.")

    res = scanner.scan_log(
        _precert_matches, "https://ct.googleapis.com/pilot", FLAGS.multi,
        write_matched_certificate,
        start_entry=FLAGS.start_entry)

    print "Scanned %d, %d matched and %d failed strict or partial parsing" % (
        res.total, res.matches, res.errors)


if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    run()



#!/usr/bin/env python
import gflags
from google.protobuf import text_format
import logging
import os
import sys
import time
from wsgiref import simple_server

from ct.dashboard import grapher
from ct.client.db import sqlite_connection as sqlitecon
from ct.client import prober
from ct.client.db import sqlite_cert_db
from ct.client.db import sqlite_log_db
from ct.client.db import sqlite_temp_db
from ct.proto import client_pb2


FLAGS = gflags.FLAGS

gflags.DEFINE_string("dashboard_host", "127.0.0.1", "Dashboard server host")
gflags.DEFINE_integer("dashboard_port", 8000, "Dashboard server port")
gflags.DEFINE_string("ctlog_config", "ct/config/logs.config",
                     "Configuration file for log servers to monitor")
# TODO(ekasper): clean up storage configuration so there's one base directory.
gflags.DEFINE_string("ct_sqlite_db", "/tmp/ct", "Location of the CT database")
gflags.DEFINE_string("ct_sqlite_cert_db", "/tmp/ct_cert", "Location of "
                     "certificate database.")
gflags.DEFINE_string("ct_sqlite_temp_dir", "/tmp/ct_tmp", "Directory for "
                     "temporary CT data.")
gflags.DEFINE_string("monitor_state_dir", "/tmp/ct_monitor",
                     "Filename prefix for monitor state. State for a given log "
                     "will be stored in a monitor_state_dir/log_id file")
gflags.DEFINE_string("log_level", "WARNING", "logging level")

class DashboardServer(object):
    template_pages = {"default", "sth"}

    """A simple web application for serving status pages."""
    def __init__(self, host, port, db, ct_server_list):
        self.host = host
        self.port = port
        self.db = db
        self.grapher = grapher.GvizGrapher()
        self.templates = {}
        self._read_templates()
        self.favicon = ""
        self._read_favicon()

    def _read_templates(self):
        """Read the HTML templates from the templates/ directory."""
        for page in DashboardServer.template_pages:
            template_file = os.path.join(os.path.dirname(__file__),
                                         "templates/%s.html" % page)
            self.templates[page] = open(template_file, "r").read()

    def _read_favicon(self):
        """Read favicon.ico."""
        favicon_file = os.path.join(os.path.dirname(__file__), "favicon.ico")
        self.favicon = open(favicon_file, "rb").read()

    def __repr__(self):
        return "%r(%r:%r)" % (self.__class__.__name__,
                              self.host, self.port)

    def __str__(self):
        return "%s(%s:%d)" % (self.__class__.__name__,
                              self.host, self.port)

    def __call__(self, environ, start_response):
        page = environ.get("PATH_INFO", "").strip("/")
        # Serve the default page when there is no match
        status = "200 OK"
        if page == "favicon.ico":
                headers = [("Content-type", "image/png")]
                start_response(status, headers)
                return self.favicon

        status = "200 OK"
        headers = [("Content-type", "text/html")]
        start_response(status, headers)
        varz = {}
        varz["current_time"] = "\nCurrent time: %s\n" % time.strftime("%c")
        if page == "sth":
            self._fill_sth_template(varz)
            return self.templates["sth"] % varz
        else:
            self._fill_default_template(varz)
            return self.templates["default"] % varz

    def _fill_default_template(self, varz):
        """Update the varz with variables required by the default template."""
        varz["pages"] = """<a href=http://%s:%d/sth>
                        The Signed Tree Head Dashboard</a>\n""" % (
            self.host, self.port)

    def _fill_sth_template(self, varz):
        # TODO(ekasper): graph all logs
        server_id = ct_server_list[0]
        varz["log"] = server_id.encode("ascii")
        one_week_ago = (time.time() - 7*24*60*60)*1000
        columns = [("sth.timestamp", "timestamp_ms", "Timestamp"),
                   ("sth.tree_size", "number", "Tree size")]
        sth_to_show = self.db.scan_latest_sth_range(server_id,
                                                    start=one_week_ago,
                                                    limit=1000)
        varz["json"] = self.grapher.make_table(sth_to_show, columns,
                                               order_by="timestamp")

    def run(self):
        """Set up the HTTP server and loop forever."""
        httpd = simple_server.make_server(self.host, self.port, self)
        logging.info("Serving on http://%s:%d" % (self.host, self.port))
        httpd.serve_forever()

if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    logging.basicConfig(level=FLAGS.log_level)

    sqlite_log_db = sqlite_log_db.SQLiteLogDB(
        sqlitecon.SQLiteConnectionManager(FLAGS.ct_sqlite_db))

    sqlite_cert_db = sqlite_cert_db.SQLiteCertDB(
        sqlitecon.SQLiteConnectionManager(FLAGS.ct_sqlite_cert_db))

    if not os.path.exists(FLAGS.ct_sqlite_temp_dir):
        logging.info("Creating directory for temporary data at %s" %
                     FLAGS.ct_sqlite_temp_dir)
        os.makedirs(FLAGS.ct_sqlite_temp_dir)

    if not os.path.exists(FLAGS.monitor_state_dir):
        logging.info("Creating directory for monitor state at %s" %
                     FLAGS.monitor_state_dir)
        os.makedirs(FLAGS.monitor_state_dir)

    sqlite_temp_db_factory = sqlite_temp_db.SQLiteTempDBFactory(
        sqlitecon.SQLiteConnectionManager(FLAGS.ct_sqlite_temp_dir + "/meta"),
                                          FLAGS.ct_sqlite_temp_dir)

    ctlogs = client_pb2.CtLogs()
    with open(FLAGS.ctlog_config, "r") as config:
        log_config = config.read()
    text_format.Merge(log_config, ctlogs)

    ct_server_list = []
    for log in ctlogs.ctlog:
        sqlite_log_db.update_log(log)
        ct_server_list.append(log.log_server)

    prober_thread = prober.ProberThread(ctlogs,
                                        sqlite_log_db,
                                        sqlite_cert_db,
                                        sqlite_temp_db_factory,
                                        FLAGS.monitor_state_dir)
    prober_thread.daemon = True
    prober_thread.start()
    server = DashboardServer(FLAGS.dashboard_host, FLAGS.dashboard_port,
                             sqlite_log_db, ct_server_list)
    server.run()



#!/usr/bin/env python

import unittest

import sys
from ct.client.db import sqlite_connection as sqlitecon
from ct.client.db import sqlite_cert_db
from ct.client.db import cert_db_test
import gflags

class SQLiteCertDBTest(unittest.TestCase, cert_db_test.CertDBTest):
    def setUp(self):
        self.database = sqlite_cert_db.SQLiteCertDB(
            sqlitecon.SQLiteConnectionManager(":memory:", keepalive=True))
    def db(self):
        return self.database

if __name__ == '__main__':
    sys.argv = gflags.FLAGS(sys.argv)
    unittest.main()



import sqlite3

from ct.client.db import database

class SQLiteConnection(object):
    """A thin wrapper around sqlite3 Connection for automatically closing the
    connection."""
    def __init__(self, db, keepalive=False):
        """Create a new connection object.
        Args:
            db:        database file, or ":memory:"
            keepalive: If True, don't close upon __exit__'ing.
        Usage:
            with SQLiteConnection(db_name) as conn:
                # conn.execute(...)
                # ...
        """
        self.__keepalive = keepalive
        # The default timeout is 5 seconds.
        # TODO(ekasper): tweak this as needed
        try:
            self.__conn = sqlite3.connect(db, timeout=600)
        # Note: The sqlite3 module does not document its error conditions so
        # it'll probably take a few iterations to get the exceptions right.
        except sqlite3.OperationalError as e:
            raise database.OperationalError(e)
        self.__conn.row_factory = sqlite3.Row

    def __repr__(self):
        return "%r(%r, keepalive=%r)" % (self.__class__.__name__, self.__db,
                                         self.__keepalive)

    def __str__(self):
        return "%s(db: %s, keepalive: %s): " % (self.__class__.__name__,
                                                self.__db, self.__keepalive)

    def __enter__(self):
        """Return the underlying raw sqlite3 Connection object."""
        self.__conn.__enter__()
        return self.__conn

    def __exit__(self, exc_type, exc_value, traceback):
        """Commit or rollback, and close the connection."""
        ret = self.__conn.__exit__(exc_type, exc_value, traceback)
        if not self.__keepalive:
            self.__conn.close()
        return ret

# Currently a very stupid manager that doesn't limit the number of connections -
# connections are simply closed and reopened every time. This could be refined
# by having the manager maintain a connection pool.
class SQLiteConnectionManager(object):
    def __init__(self, db, keepalive=False):
        """Connection manager for a SQLite database.
        Args:
            db:        database file, or ":memory:"
            keepalive: If True, maintains a single open connection.
                       If False, returns a new connection to be created for each
                       call. keepalive=True is not thread-safe.
        """
        self.__db = db
        self.__keepalive = keepalive
        self.__conn = None
        if keepalive:
            self.__conn = SQLiteConnection(self.__db, keepalive=True)

    def __repr__(self):
        return "%r(%r, keepalive=%r)" % (self.__class__.__name__, self.__db,
                                         self.__keepalive)

    def __str__(self):
        return "%s(db: %s, keepalive: %s): " % (self.__class__.__name__,
                                                self.__db, self.__keepalive)

    @property
    def db_name(self):
        return self.__db

    def get_connection(self):
        """In keepalive mode, return the single persistent connection.
        Else return a new connection instance."""
        if self.__keepalive:
            return self.__conn
        else:
            return SQLiteConnection(self.__db, keepalive=False)



import abc
import hashlib


class CertDB(object):
    """Database interface for storing X509 certificate information."""
    __metaclass__ = abc.ABCMeta

    @staticmethod
    def sha256_hash(der_cert):
        return hashlib.sha256(der_cert).digest()

    @abc.abstractmethod
    def store_cert_desc(self, cert_desc, index, log_key):
        """Stores a certificate using its description.

        Args:
            cert:          CertificateDescription
            index:         position in log
            log_key:       log id in LogDB"""

    @abc.abstractmethod
    def store_certs_desc(self, certs, log_key):
        """Store certificates using its descriptions.

        Batched version of store_cert_desc.

        Args:
            certs:         iterable of (CertificateDescription, index) tuples
            log_key:       log id in LogDB"""

    @abc.abstractmethod
    def get_cert_by_sha256_hash(self, cert_sha256_hash):
        """Fetch a certificate with a matching SHA256 hash
        Args:
            cert_sha256_hash: the SHA256 hash of the certificate
        Returns:
            A DER-encoded certificate, or None if the cert is not found."""

    @abc.abstractmethod
    def scan_certs(self, limit=0):
        """Scan all certificates.
        Args:
            limit:        maximum number of entries to yield. Default is no
                          limit.
        Yields:
            DER-encoded certificates."""

    @abc.abstractmethod
    def scan_certs_by_subject(self, subject_name, limit=0):
        """Scan certificates matching a subject name.
        Args:
            subject_name: a subject name, usually a domain. A scan for
                          example.com returns certificates for www.example.com,
                          *.example.com, test.mail.example.com, etc. Similarly
                          'com' can be used to look for all .com certificates.
                          Wildcards are treated as literal characters: a search
                          for *.example.com returns certificates for
                          *.example.com but not for mail.example.com and vice
                          versa.
                          Name may also be a common name rather than a DNS name,
                          e.g., "Trustworthy Certificate Authority".
            limit:        maximum number of entries to yield. Default is no
                          limit.
        Yields:
            DER-encoded certificates."""



class Error(Exception):
    pass

class KeyError(Error):
    """Raised when key constraints are violated."""
    pass

class OperationalError(Error):
    """Raised when a database operation fails, e.g., because of a timeout.
    May be raised by all Database operations including __init__"""
    pass



#!/usr/bin/env python
# coding=utf-8
import unittest

import time
import sys
from ct.client.db import cert_desc
from ct.crypto import cert
from ct.test import test_config
from ct.test import time_utils
import gflags

CERT = cert.Certificate.from_der_file(
        test_config.get_test_file_path("google_cert.der"))
CA_CERT = cert.Certificate.from_pem_file(
        test_config.get_test_file_path("verisign_intermediate.pem"))
DSA_SHA256_CERT = cert.Certificate.from_der_file(
        test_config.get_test_file_path("dsa_with_sha256.der"))
BAD_UTF8_CERT = cert.Certificate.from_pem_file(
        test_config.get_test_file_path("cert_bad_utf8_subject.pem"))
DOMAIN_IN_ISSUER_CERT = cert.Certificate.from_pem_file(
        test_config.get_test_file_path("domain_in_issuer.pem"))
DOMAIN_IN_O_COMPONENT = cert.Certificate.from_pem_file(
        test_config.get_test_file_path("domain_in_o_component.pem"))

class CertificateDescriptionTest(unittest.TestCase):
    def assert_description_subject_matches_source(self, proto, source):
        subject = [(att.type, att.value) for att in proto.subject]
        cert_subject = [(type_.short_name,
                     cert_desc.to_unicode('.'.join(
                             cert_desc.process_name(value.human_readable()))))
                    for type_, value in source.subject()]
        self.assertItemsEqual(cert_subject, subject)

    def assert_description_issuer_matches_source(self, proto, source):
        issuer = [(att.type, att.value) for att in proto.issuer]
        cert_issuer = [(type_.short_name,
                     cert_desc.to_unicode('.'.join(
                             cert_desc.process_name(value.human_readable()))))
                    for type_, value in source.issuer()]
        self.assertItemsEqual(cert_issuer, issuer)

    def assert_description_alt_subject_names_match_source(self, proto, source):
        subject_alternative_names = [(att.type, att.value)
                                     for att in proto.subject_alternative_names]
        cert_subject_alternative_names = [(san.component_key(),
                                           cert_desc.to_unicode('.'.join(
                                            cert_desc.process_name(
                                     san.component_value().human_readable()))))
                    for san in source.subject_alternative_names()]
        self.assertItemsEqual(cert_subject_alternative_names,
                              subject_alternative_names)

    def assert_description_serial_number_matches_source(self, proto, source):
        self.assertEqual(proto.serial_number,
                         str(source.serial_number().human_readable()
                             .upper().replace(':', '')))

    def assert_description_validity_dates_match_source(self, proto, source):
        self.assertEqual(time.gmtime(proto.validity.not_before / 1000),
                         source.not_before())
        self.assertEqual(time.gmtime(proto.validity.not_after / 1000),
                         source.not_after())

    def assert_description_signature_matches_source(self, proto, source):
        self.assertEqual(proto.tbs_signature.algorithm_id,
                         source.signature()["algorithm"].long_name)
        self.assertEqual(proto.cert_signature.algorithm_id,
                         source.signature_algorithm()["algorithm"].long_name)
        self.assertEqual(proto.tbs_signature.algorithm_id,
                         proto.cert_signature.algorithm_id)

        if source.signature()["parameters"]:
            self.assertEqual(proto.tbs_signature.parameters,
                             source.signature()["parameters"])
        else:
            self.assertFalse(proto.tbs_signature.HasField('parameters'))

        if source.signature_algorithm()["parameters"]:
            self.assertEqual(proto.cert_signature.parameters,
                             source.signature_algorithm()["parameters"])
        else:
            self.assertFalse(proto.cert_signature.HasField('parameters'))

        self.assertEqual(proto.tbs_signature.parameters,
                         proto.cert_signature.parameters)

    def assert_description_matches_source(self, source, expect_ca_true):
        proto = cert_desc.from_cert(source)

        self.assertEqual(proto.der, source.to_der())
        self.assert_description_subject_matches_source(proto, source)
        self.assert_description_issuer_matches_source(proto, source)
        self.assert_description_alt_subject_names_match_source(proto, source)
        self.assertEqual(proto.version, str(source.version().value))
        self.assert_description_serial_number_matches_source(proto, source)
        self.assert_description_validity_dates_match_source(proto, source)
        self.assert_description_signature_matches_source(proto, source)
        self.assertEqual(proto.basic_constraint_ca, expect_ca_true)

    def test_from_cert(self):
        cert = CERT
        is_ca_cert = False

        with time_utils.timezone("UTC"):
            self.assert_description_matches_source(cert, is_ca_cert)

        # Test in non-UTC timezones, to detect timezone issues
        with time_utils.timezone("America/Los_Angeles"):
            self.assert_description_matches_source(cert, is_ca_cert)

        with time_utils.timezone("Asia/Shanghai"):
            self.assert_description_matches_source(cert, is_ca_cert)

    def test_from_cert_with_dsa_sha256_cert(self):
        cert = DSA_SHA256_CERT
        is_ca_cert = False

        with time_utils.timezone("UTC"):
            self.assert_description_matches_source(cert, is_ca_cert)

        # Test in non-UTC timezones, to detect timezone issues
        with time_utils.timezone("America/Los_Angeles"):
            self.assert_description_matches_source(cert, is_ca_cert)

        with time_utils.timezone("Asia/Shanghai"):
            self.assert_description_matches_source(cert, is_ca_cert)

    def test_from_cert_with_ca_cert(self):
        cert = CA_CERT
        is_ca_cert = True

        with time_utils.timezone("UTC"):
            self.assert_description_matches_source(cert, is_ca_cert)

        # Test in non-UTC timezones, to detect timezone issues
        with time_utils.timezone("America/Los_Angeles"):
            self.assert_description_matches_source(cert, is_ca_cert)

        with time_utils.timezone("Asia/Shanghai"):
            self.assert_description_matches_source(cert, is_ca_cert)

    def test_cert_with_mangled_utf8(self):
        cert = BAD_UTF8_CERT
        is_ca_cert = False

        with time_utils.timezone("UTC"):
            self.assert_description_matches_source(cert, is_ca_cert)

        # Test in non-UTC timezones, to detect timezone issues
        with time_utils.timezone("America/Los_Angeles"):
            self.assert_description_matches_source(cert, is_ca_cert)

        with time_utils.timezone("Asia/Shanghai"):
            self.assert_description_matches_source(cert, is_ca_cert)

        proto = cert_desc.from_cert(cert)

    def test_does_not_reverse_domain_names_in_ou(self):
        ou = [t for t in cert_desc.from_cert(DOMAIN_IN_ISSUER_CERT).issuer
              if t.type == "OU"][0]
        self.assertEqual("www.digicert.com", ou.value)

    def test_does_not_reverse_domain_names_in_o_component(self):
        ou = [t for t in cert_desc.from_cert(DOMAIN_IN_O_COMPONENT).subject
              if t.type == "O"][0]
        self.assertEqual("go-greenevents.com", ou.value)

    def test_process_value(self):
        self.assertEqual(["London"], cert_desc.process_name("London"))
        self.assertEqual(["Bob Smith"], cert_desc.process_name("Bob Smith"))
        self.assertEqual(["com", "googleapis", "ct"],
                         cert_desc.process_name("ct.googleapis.com"))
        self.assertEqual(["com", "github"],
                         cert_desc.process_name("gItHuB.CoM"))
        # These two are unfortunate outcomes:
        # 1. single-word hostnames are indistinguishable from single-word CN
        # terms like State, City, Organization
        self.assertEqual(["LOCALhost"], cert_desc.process_name("LOCALhost"))
        # 2. IP addresses should perhaps not be reversed like hostnames are
        self.assertEqual(["1", "0", "168", "192"],
                         cert_desc.process_name("192.168.0.1"))

    def test_to_unicode(self):
        self.assertEqual(u"foobar", cert_desc.to_unicode("foobar"))
        # The given string is encoded using ISO-8859-1, not UTF-8.
        # Assuming it is UTF-8 yields invalid Unicode \uDBE0.
        self.assertNotEqual(u"R\uDBE0S", cert_desc.to_unicode("R\xED\xAF\xA0S"))
        # Detecting the failure and retrying as ISO-8859-1.
        self.assertEqual(u"R\u00ED\u00AF\u00A0S",
                         cert_desc.to_unicode("R\xED\xAF\xA0S"))


if __name__ == "__main__":
    sys.argv = gflags.FLAGS(sys.argv)
    unittest.main()



"""Metadata for Extended Validation certificates and roots"""

from collections import defaultdict

from ct.crypto.asn1 import oid

# Most root fingerprints (as well as EV policy OIDs) are from Chrome:
# https://code.google.com/p/chromium/codesearch#chromium/src/net/cert/ev_root_ca_metadata.cc
# In many cases the chains for EV certificates logged chain to an obsolete root
# which was replaced since logging.
# Fingerprints for these old roots (together with evidence of their
# origin) were added.
_EV_ROOTS = (
    # AC Camerfirma S.A. Chambers of Commerce Root - 2008
    # https://www.camerfirma.com
    (
        "786a74ac76ab147f9c6a3050ba9ea87efe9ace3c",
        # AC Camerfirma uses the last two arcs to track how the private key is
        # managed - the effective verification policy is the same.
        ("1.3.6.1.4.1.17326.10.14.2.1.2",
         "1.3.6.1.4.1.17326.10.14.2.2.2")
        ),

    # AC Camerfirma S.A. Global Chambersign Root - 2008
    # https://server2.camerfirma.com:8082
    (
        "4abdeeec950d359c89aec752a12c5b29f6d6aa0c",
        # AC Camerfirma uses the last two arcs to track how the private key is
        # managed - the effective verification policy is the same.
        ("1.3.6.1.4.1.17326.10.8.12.1.2",
         "1.3.6.1.4.1.17326.10.8.12.2.2")
        ),

    # AddTrust External CA Root
    # https://addtrustexternalcaroot-ev.comodoca.com
    (
        "02faf3e291435468607857694df5e45b68851868",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",
         # This is the Network Solutions EV OID. However, this root
         # cross-certifies NetSol and so we need it here too.
         "1.3.6.1.4.1.782.1.2.1.8.1")
        ),

    # AffirmTrust Commercial
    # https://commercial.affirmtrust.com/
    (
        "f9b5b632455f9cbeec575f80dce96e2cc7b278b7",
        ("1.3.6.1.4.1.34697.2.1",),
        ),

    # AffirmTrust Networking
    # https://networking.affirmtrust.com:4431
    (
        "293621028b20ed02f566c532d1d6ed909f45002f",
        ("1.3.6.1.4.1.34697.2.2",),
        ),

    # AffirmTrust Premium
    # https://premium.affirmtrust.com:4432/
    (
        "d8a6332ce0036fb185f6634f7d6a066526322827",
        ("1.3.6.1.4.1.34697.2.3",),
        ),

    # AffirmTrust Premium ECC
    # https://premiumecc.affirmtrust.com:4433/
    (
        "b8236b002f1d16865301556c11a437caebffc3bb",
        ("1.3.6.1.4.1.34697.2.4",),
        ),

    # Buypass Class 3 CA 1
    # https://valid.evident.ca13.ssl.buypass.no/
    (
        "61573A11DF0ED87ED5926522EAD056D744B32371",
        ("2.16.578.1.26.1.3.3",),
        ),

    # Buypass Class 3 Root CA
    # https://valid.evident.ca23.ssl.buypass.no/
    (
        "DAFAF7FA6684EC068F1450BDC7C281A5BCA96457",
        ("2.16.578.1.26.1.3.3",),
        ),

    # CertPlus Class 2 Primary CA (KEYNECTIS)
    # https://www.keynectis.com/
    (
        "74207441729cdd92ec7931d823108dc28192e2bb",
        ("1.3.6.1.4.1.22234.2.5.2.3.1",),
        ),

    # Certum Trusted Network CA
    # https://juice.certum.pl/
    (
        "07e032e020b72c3f192f0628a2593a19a70f069e",
        ("1.2.616.1.113527.2.5.1.1",),
        ),

    # Certum CA
    # Was recognized by Microsoft at some point:
    # http://social.technet.microsoft.com/wiki/contents/articles/2592.windows-root-certificate-program-members-list-all-cas.aspx
     (
         "6252dc40f71143a22fde9ef7348e064251b18118",
         ("1.2.616.1.113527.2.5.1.1",),
         ),

    # China Internet Network Information Center EV Certificates Root
    # https://evdemo.cnnic.cn/
    (
        "4F99AA93FB2BD13726A1994ACE7FF005F2935D1E",
        ("1.3.6.1.4.1.29836.1.10",),
        ),

    # COMODO Certification Authority
    # https://secure.comodo.com/
    (
        "6631bf9ef74f9eb6c9d5a60cba6abed1f7bdef7b",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",),
        ),

    # Some more Comodo, see
    # https://code.google.com/p/android/issues/detail?id=54479 and
    # https://code.google.com/p/chromium/issues/detail?id=231900
    (
        "afe5d244a8d1194230ff479fe2f897bbcd7a8cb4",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",),
        ),
    (
        "2b8f1b57330dbba2d07a6c51f70ee90ddab9ad8e",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",),
        ),
    (
        "d1cbca5db2d52a7f693b674de5f05a1d0c957df0",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",),
        ),

    # COMODO Certification Authority (reissued certificate with NotBefore of Jan
    # 1 00:00:00 2011 GMT)
    # https://secure.comodo.com/
    (
        "ee869387fffd8349ab5ad14322588789a457b012",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",),
        ),

    # COMODO ECC Certification Authority
    # https://comodoecccertificationauthority-ev.comodoca.com/
    (
        "9f744e9f2b4dbaec0f312c50b6563b8e2d93c311",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",),
        ),

    # Cybertrust Global Root
    # https://evup.cybertrust.ne.jp/ctj-ev-upgrader/evseal.gif
    (
        "5f43e5b1bff8788cac1cc7ca4a9ac6222bcc34c6",
        ("1.3.6.1.4.1.6334.1.100.1",),
        ),

    # Baltimore CyberTrust Root
    (
        # See https://code.google.com/p/android/issues/detail?id=9269
         "d4de20d05e66fc53fe1a50882c78db2852cae474",
        # # Cybertrust Global Root
         ("1.3.6.1.4.1.6334.1.100.1",
          # DigiCert
          "2.16.840.1.114412.2.1"),
         ),

    # DigiCert High Assurance EV Root CA
    # https://www.digicert.com
    (
        "5fb7ee0633e259dbad0c4c9ae6d38f1a61c7dc25",
        ("2.16.840.1.114412.2.1",),
        ),

    # D-TRUST Root Class 3 CA 2 EV 2009
    # https://certdemo-ev-valid.ssl.d-trust.net/
    (
        "96c91b0b95b4109842fad0d82279fe60fab91683",
        ("1.3.6.1.4.1.4788.2.202.1",),
        ),

    # Entrust.net Secure Server Certification Authority
    # https://www.entrust.net/
    (
        "99a69be61afe886b4d2b82007cb854fc317e1539",
        ("2.16.840.1.114028.10.1.2",
        # DigiCert
        # "2.16.840.1.114412.2.1",
        # SecureTrust
        # "2.16.840.1.114404.1.1.2.4.1"
        ),
    ),

    # Entrust Root Certification Authority
    # https://www.entrust.net/
    (
        "b31eb1b740e36c8402dadc37d44df5d4674952f9",
        ("2.16.840.1.114028.10.1.2",),
        ),
    # Another Entrust root - Entrust Root Certification Authority - G2
    # Included in Mac OS X at the very least
    (
        "8cf427fd790c3ad166068de81e57efbb932272d4",
        ("2.16.840.1.114028.10.1.2",),
        ),
    # Old Entrust root
    # See https://bugzilla.mozilla.org/show_bug.cgi?id=849833
    (
        "801d62d07b449d5c5c035c98ea61fa443c2a58fe",
        ("2.16.840.1.114028.10.1.2",),
        ),

    # Entrust-DigiCert cross-signed intermediate
    # Entrust.net Certification Authority (2048)
     (
         # Replacement root, see
         # https://bugzilla.mozilla.org/show_bug.cgi?id=849833
         "503006091d97d4f5ae39f7cbe7927d7d652d3431",
         ("2.16.840.1.114412.2.1",),
         ),

    # Equifax Secure Certificate Authority (GeoTrust)
    # https://www.geotrust.com/
    (
        "d23209ad23d314232174e40d7f9d62139786633a",
        ("1.3.6.1.4.1.14370.1.6",),
        ),

    # E-Tugra Certification Authority
    # https://sslev.e-tugra.com.tr
    (
        "51C6E70849066EF392D45CA00D6DA3628FC35239",
        ("2.16.792.3.0.4.1.1.4",),
        ),

    # GeoTrust Primary Certification Authority
    # https://www.geotrust.com/
    (
        "323c118e1bf7b8b65254e2e2100dd6029037f096",
        ("1.3.6.1.4.1.14370.1.6",),
        ),

    # GeoTrust Primary Certification Authority - G2
    (
        "8d1784d537f3037dec70fe578b519a99e610d7b0",
        ("1.3.6.1.4.1.14370.1.6",),
        ),

    # GeoTrust Primary Certification Authority - G3
    (
        "039eedb80be7a03c6953893b20d2d9323a4c2afd",
        ("1.3.6.1.4.1.14370.1.6",),
        ),

    # GlobalSign Root CA - R2
    # https://www.globalsign.com/
    (
        "75e0abb6138512271c04f85fddde38e4b7242efe",
        ("1.3.6.1.4.1.4146.1.1",),
        ),

    # GlobalSign Root CA
    (
        "b1bc968bd4f49d622aa89a81f2150152a41d829c",
        ("1.3.6.1.4.1.4146.1.1",),
        ),

    # GlobalSign Root CA - R3
    # https://2029.globalsign.com/
    (
        "d69b561148f01c77c54578c10926df5b856976ad",
        ("1.3.6.1.4.1.4146.1.1",),
        ),
    # GlobalSign Root CA - phased out
    # See https://2014.globalsign.com/
    (
        "2f173f7de99667afa57af80aa2d1b12fac830338",
        ("1.3.6.1.4.1.4146.1.1",),
        ),


    # Go Daddy Class 2 Certification Authority
    # https://www.godaddy.com/
    (
        "2796bae63f1801e277261ba0d77770028f20eee4",
        ("2.16.840.1.114413.1.7.23.3",),
        ),

    # Go Daddy Root Certificate Authority - G2
    # https://valid.gdig2.catest.godaddy.com/
    (
        "47beabc922eae80e78783462a79f45c254fde68b",
        ("2.16.840.1.114413.1.7.23.3",),
        ),

    # GTE CyberTrust Global Root
    # https://www.cybertrust.ne.jp/
    (
        "97817950d81c9670cc34d809cf794431367ef474",
        ("1.3.6.1.4.1.6334.1.100.1",
         # DigiCert
         # "2.16.840.1.114412.2.1"
         ),
        ),
    # Izenpe.com - SHA256 root
    # The first OID is for businesses and the second for government entities.
    # These are the test sites, respectively:
    # https://servicios.izenpe.com
    # https://servicios1.izenpe.com
    (
        "2f783d255218a74a653971b52ca29c45156fe919",
    ("1.3.6.1.4.1.14777.6.1.1", "1.3.6.1.4.1.14777.6.1.2"),
    ),

    # Izenpe.com - SHA1 root
    # Windows XP finds this, SHA1, root instead. The policy OIDs are the same as
    # for the SHA256 root, above.
    (
        "30779e9315022e94856a3ff8bcf815b082f9aefd",
        ("1.3.6.1.4.1.14777.6.1.1", "1.3.6.1.4.1.14777.6.1.2"),
        ),

    # Network Solutions Certificate Authority
    # https://www.networksolutions.com/website-packages/index.jsp
    (
        "74f8a3c3efe7b390064b83903c21646020e5dfce",
        ("1.3.6.1.4.1.782.1.2.1.8.1",),
        ),

    # Network Solutions Certificate Authority (reissued certificate with
    # NotBefore of Jan  1 00:00:00 2011 GMT).
    # https://www.networksolutions.com/website-packages/index.jsp
    (
        "71899a67bf33af31befdc071f8f733b183856332",
        ("1.3.6.1.4.1.782.1.2.1.8.1",),
        ),

    # QuoVadis Root CA 2
    # https://www.quovadis.bm/
    (
        "ca3afbcf1240364b44b216208880483919937cf7",
        ("1.3.6.1.4.1.8024.0.2.100.1.2",),
        ),

    # QuoVadis Root (cross-signing QuoVadis Root CA 2)
    # https://www.quovadis.bm/
    # Old certificate, see:
    # http://www.quovadisglobal.co.uk/~/media/Files/Repository/QV_RCA1_RCA3_CPCPS_V4_5.ashx
     (
         "de3f40bd5093d39b6c60f6dabc076201008976c9",
         ("1.3.6.1.4.1.8024.0.2.100.1.2",),
         ),


    # SecureTrust CA, SecureTrust Corporation
    # https://www.securetrust.com
    # https://www.trustwave.com/
    (
        "8782c6c304353bcfd29692d2593e7d44d934ff11",
        ("2.16.840.1.114404.1.1.2.4.1",),
        ),

    # Secure Global CA, SecureTrust Corporation
    (
        "3a44735ae581901f248661461e3b9cc45ff53a1b",
        ("2.16.840.1.114404.1.1.2.4.1",),
        ),

    # Security Communication RootCA1
    # https://www.secomtrust.net/contact/form.html
    (
        "36b12b49f9819ed74c9ebc380fc6568f5dacb2f7",
        ("1.2.392.200091.100.721.1",),
        ),

    # Security Communication EV RootCA1
    # https://www.secomtrust.net/contact/form.html
    (
        "feb8c432dcf9769aceae3dd8908ffd288665647d",
        ("1.2.392.200091.100.721.1",),
        ),

    # StartCom Certification Authority
    # https://www.startssl.com/
    (
        "3e2bf7f2031b96f38ce6c4d8a85d3e2d58476a0f",
        ("1.3.6.1.4.1.23223.1.1.1",),
        ),

    # StartCom Certification Authority G2
    # https://www.startssl.com/
    # Renewed StartCom root certs, see:
    # https://bugzilla.mozilla.org/show_bug.cgi?id=751954
     (
         "31f1fd68226320eec63b3f9dea4a3e537c7c3917",
         ("1.3.6.1.4.1.23223.1.1.1",),
         ),

    # StartCom Certification Authority (SHA2)
    # https://www.startssl.com/
    # Renewed StartCom root certs, see:
    # https://bugzilla.mozilla.org/show_bug.cgi?id=751954
     (
         "a3f1333fe242bfcfc5d14e8f394298406810d1a0",
         ("1.3.6.1.4.1.23223.1.1.1",),
         ),

    # Starfield Class 2 Certification Authority
    # https://www.starfieldtech.com/
    (
        "ad7e1c28b064ef8f6003402014c3d0e3370eb58a",
        ("2.16.840.1.114414.1.7.23.3",),
        ),

    # Starfield Root Certificate Authority - G2
    # https://valid.sfig2.catest.starfieldtech.com/
    (
        "b51c067cee2b0c3df855ab2d92f4fe39d4e70f0e",
        ("2.16.840.1.114414.1.7.23.3",),
        ),

    # Starfield Services Root Certificate Authority - G2
    # https://valid.sfsg2.catest.starfieldtech.com/
    (
        "925a8f8d2c6d04e0665f596aff22d863e8256f3f",
        ("2.16.840.1.114414.1.7.24.3",),
        ),

    # SwissSign Gold CA - G2
    # https://testevg2.swisssign.net/
    (
        "d8c5388ab7301b1b6ed47ae645253a6f9f1a2761",
        ("2.16.756.1.89.1.2.1.1",
         # Affirmtrust cross-sign
         # "1.3.6.1.4.1.34697.2.2"
         ),
        ),

    # Thawte Premium Server CA
    # https://www.thawte.com/
    (
        "627f8d7827656399d27d7f9044c9feb3f33efa9a",
        ("2.16.840.1.113733.1.7.48.1",),
        ),

    # thawte Primary Root CA
    # https://www.thawte.com/
    (
        "91c6d6ee3e8ac86384e548c299295c756c817b81",
        ("2.16.840.1.113733.1.7.48.1",),
        ),

    # thawte Primary Root CA - G2
    (
        "aadbbc22238fc401a127bb38ddf41ddb089ef012",
        ("2.16.840.1.113733.1.7.48.1",),
        ),

    # thawte Primary Root CA - G3
    (
        "f18b538d1be903b6a6f056435b171589caf36bf2",
        ("2.16.840.1.113733.1.7.48.1",),
        ),
    # Thawte Root 2
    # Thawte Premium Server CA - http://www.thawte.com/roots/
    # Transition certificate, see Root 2 under http://www.thawte.com/roots/
    (
        "e0ab059420725493056062023670f7cd2efc6666",
        ("2.16.840.1.113733.1.7.48.1",),
        ),

    # TWCA Global Root CA
    # https://evssldemo3.twca.com.tw/index.html
    (
        "9CBB4853F6A4F6D352A4E83252556013F5ADAF65",
        ("1.3.6.1.4.1.40869.1.1.22.3",),
        ),

    # TWCA Root Certification Authority
    # https://evssldemo.twca.com.tw/index.html
    (
        "cf9e876dd3ebfc422697a3b5a37aa076a9062348",
        ("1.3.6.1.4.1.40869.1.1.22.3",),
        ),

    # T-TeleSec GlobalRoot Class 3
    # http://www.telesec.de/ / https://root-class3.test.telesec.de/
    (
        "55a6723ecbf2eccdc3237470199d2abe11e381d1",
        ("1.3.6.1.4.1.7879.13.24.1",),
        ),

    # UTN - DATACorp SGC
    (
        "58119f0e128287ea50fdd987456f4f78dcfad6d4",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",),
        ),

    # UTN-USERFirst-Hardware
    (
        "0483ed3399ac3608058722edbc5e4600e3bef9d7",
        ("1.3.6.1.4.1.6449.1.2.1.5.1",
         # This is the Network Solutions EV OID. However, this root
         # cross-certifies NetSol and so we need it here too.
         "1.3.6.1.4.1.782.1.2.1.8.1"),
        ),

    # ValiCert Class 2 Policy Validation Authority
    (
        "317a2ad07f2b335ef5a1c34e4b57e8b7d8f1fca6",
        ("2.16.840.1.114413.1.7.23.3", "2.16.840.1.114414.1.7.23.3"),
        ),

    # ValiCert Class 1 Policy Validation Authority
    # Old 1024 bit root, see
    # https://developer.mozilla.org/en-US/docs/Mozilla/Projects/NSS/NSS_3.16.3_release_notes
    (
        "e5df743cb601c49b9843dcab8ce86a81109fe48e",
        # SECOM
        ("1.2.392.200091.100.721.1",),
        ),

    # VeriSign Class 3 Public Primary Certification Authority (MD2)
    # https://www.verisign.com/
    (
        "742c3192e607e424eb4549542be1bbc53e6174e2",
        ("2.16.840.1.113733.1.7.23.6",),
        ),

    # VeriSign Class 3 Public Primary Certification Authority (SHA1)
    # https://www.verisign.com/
    # Legacy certificate, see Root 2 under
    # http://www.symantec.com/page.jsp?id=roots
    (
        "a1db6393916f17e4185509400415c70240b0ae6b",
        ("2.16.840.1.113733.1.7.23.6",),
        ),


    # VeriSign Class 3 Public Primary Certification Authority - G4
    (
        "22D5D8DF8F0231D18DF79DB7CF8A2D64C93F6C3A",
        ("2.16.840.1.113733.1.7.23.6",),
        ),

    # VeriSign Class 3 Public Primary Certification Authority - G5
    # https://www.verisign.com/
    (
        "4eb6d578499b1ccf5f581ead56be3d9b6744a5e5",
        ("2.16.840.1.113733.1.7.23.6",),
        ),

    # VeriSign Universal Root Certification Authority
    (
        "3679ca35668772304d30a5fb873b0fa77bb70d54",
        ("2.16.840.1.113733.1.7.23.6",),
        ),

    # Wells Fargo WellsSecure Public Root Certificate Authority
    # https://nerys.wellsfargo.com/test.html
    (
        "e7b4f69d61ec9069db7e90a7401a3cf47d4fe8ee",
        ("2.16.840.1.114171.500.9",),
        ),

    # XRamp Global Certification Authority
    (
        "b80186d1eb9c86a54104cf3054f34c52b7e558c6",
        ("2.16.840.1.114404.1.1.2.4.1",),
        ),

    # Pending inclusion in Chrome
    # Swisscom Root EV CA 2
    (
        "e7a19029d3d552dc0d0fc692d3ea880d152e1a6b",
        ("2.16.756.1.83.21.0",),
        ),
    # Buypass Class 3 Root CA
    (
        "dafaf7fa6684ec068f1450bdc7c281a5bca96457",
        ("2.16.578.1.26.1.3.3",),
        ),
    # Autoridad de Certificacion Firmaprofesional CIF A62634068
    (
        "aec5fb3fc8e1bfc4e54f03075a9ae800b7f7b6fa",
        ("1.3.6.1.4.1.13177.10.1.3.10",),
        ),
    # TURKTRUST Elektronik Sertifika Hizmet Saglayicisi
    (
        "f17f6fb631dc99e3a3c87ffe1cf1811088d96033",
        ("2.16.792.3.0.3.1.1.5",),
        ),
    # Actalis Authentication Root CA
    (
        "f373b387065a28848af2f34ace192bddc78e9cac",
        ("1.3.159.1.17.1",),
        ),
    # QuoVadis Root CA 2 G3
    (
        "093c61f38b8bdc7d55df7538020500e125f5c836",
        ("1.3.6.1.4.1.8024.0.2.100.1.2",),
        )
)
####### Added #######
## Entrust - DigiCert x2
## GTE CyberTrust - Digicert
## VeriSign SHA1
## StartCom SHA2, StartCom G2
## SwissSign - AffirmTrust (vice versa: chain terminating at AffirmTrust
##                          includes SwissSign policy).
## QuoVadis - QuoVadis (EV on FF and not on Chrome)
## Entrust - SecureTrust
## Certum CA - Certum Trusted Network CA
## Valicert Class 1 - SECOM
## Baltimore Cybertrust Root - Cybertrust Global Root, DigiCert
## Swisscom Root EV CA 2
## Buypass Class 3 Root CA
## Autoridad de Certificacion Firmaprofesional CIF A62634068
## TURKTRUST Elektronik Sertifika Hizmet Saglayicisi
## Actalis Authentication Root CA
## QuoVadis Root CA 2 G3

EV_POLICIES = defaultdict(set)
EV_ROOTS = defaultdict(set)

for chash, policies in _EV_ROOTS:
    chash_raw = chash.decode("hex")
    assert len(chash_raw) == 20
    for policy in policies:
        EV_POLICIES[oid.ObjectIdentifier(value=policy)].add(chash_raw)
        EV_ROOTS[chash_raw].add(policy)



#!/usr/bin/env python
"""Parse and print the list of logs, after validating signature."""

import base64
import hashlib
import json
import os
import sys
import time

import gflags
import jsonschema
import M2Crypto

from cpp_generator import generate_cpp_header
from java_generator import generate_java_source
from openssl_generator import generate_openssl_conf

FLAGS = gflags.FLAGS

gflags.DEFINE_string("log_list", None, "Logs list file to parse and print.")
gflags.MarkFlagAsRequired("log_list")
gflags.DEFINE_string("signature", None, "Signature file over the list of logs.")
gflags.DEFINE_string("signer_key", None, "Public key of the log list signer.")
gflags.DEFINE_string("log_list_schema",
                     os.path.join(os.path.dirname(sys.argv[0]),
                                  "data", "log_list_schema.json"),
                     "JSON schema for the list of logs.")
gflags.DEFINE_string("header_output", None,
                     "If specifed, generates C++ code for Chromium.")
gflags.DEFINE_string("java_output", None,
                     "If specifed, generates Java code.")
gflags.DEFINE_string("java_class", "org.conscrypt.ct.KnownLogs",
                     "Fully qualified name of the generated class.")
gflags.DEFINE_string("openssl_output", None,
                     "If specified, generates a CONF file for OpenSSL.")
gflags.DEFINE_boolean("skip_signature_check", False,
                     "Skip signature check (only validate schema).")


def is_log_list_valid(json_log_list, schema_file):
    try:
        jsonschema.validate(
            json_log_list,
            json.load(open(schema_file, "rb")))
        return True
    except jsonschema.exceptions.ValidationError as e:
        print e
        return False
    return False


def is_signature_valid(log_list_data, signature_file, public_key_file):
    loaded_pubkey = M2Crypto.RSA.load_pub_key(public_key_file)
    pubkey = M2Crypto.EVP.PKey()
    pubkey.assign_rsa(loaded_pubkey)
    pubkey.reset_context(md="sha256")
    pubkey.verify_init()
    pubkey.verify_update(log_list_data)
    return pubkey.verify_final(open(signature_file, "rb").read())


def print_formatted_log_list(json_log_list):
    operator_id_to_name = dict(
        [(o["id"], o["name"]) for o in json_log_list["operators"]])

    for log_info in json_log_list["logs"]:
        print "%s:" % log_info["description"]
        log_operators = [
            operator_id_to_name[i].encode("utf-8")
            for i in log_info["operated_by"]]
        print "  Operated by %s and has MMD of %f hours" % (
            ", ".join(log_operators),
            log_info["maximum_merge_delay"] / (60.0 ** 2))
        print "  At: %s" % (log_info["url"])
        key = base64.decodestring(log_info["key"])
        hasher = hashlib.sha256()
        hasher.update(key)
        key_hash = hasher.digest()
        print "  Key ID: %s" % (base64.encodestring(key_hash)),
        if "final_sth" in log_info:
            final_sth = log_info["final_sth"]
            print "  Log is frozen as of %s, final tree size %d" % (
                time.asctime(time.gmtime(final_sth["timestamp"] / 1000.0)),
                final_sth["tree_size"])
        print "-" * 80

def run():
    with open(FLAGS.log_list, "rb") as f:
        json_data = f.read()

    if (not FLAGS.skip_signature_check) and not is_signature_valid(
        json_data, FLAGS.signature, FLAGS.signer_key):
        print "ERROR: Signature over list of logs is not valid, not proceeding."
        sys.exit(1)

    parsed_json = json.loads(json_data)
    if not is_log_list_valid(parsed_json, FLAGS.log_list_schema):
        print "ERROR: Log list is signed but does not conform to the schema."
        sys.exit(2)
    if FLAGS.header_output:
        generate_cpp_header(parsed_json, FLAGS.header_output)
    if FLAGS.java_output:
        generate_java_source(parsed_json, FLAGS.java_output, FLAGS.java_class)
    if FLAGS.openssl_output:
        generate_openssl_conf(parsed_json, FLAGS.openssl_output)

    if not FLAGS.header_output and not FLAGS.java_output:
        print_formatted_log_list(parsed_json)


if __name__ == "__main__":
    sys.argv = FLAGS(sys.argv)
    run()
